var SV = window.SV || {};
SV.AdvancedBBCode = SV.AdvancedBBCode || {};

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined)
{
    "use strict";

    SV.AdvancedBBCode.DialogForm = XF.Element.newHandler({
        options: {
            output: '[name="bb_code_output"]',
            bbCode: null,
            oldFormat: null,
            newLines: true,
            singleOption: false
        },

        $output: null,

        init: function ()
        {
            if (!this.options.bbCode)
            {
                console.error('Invalid bb code provided.');
                return;
            }

            var $output = this.$target.find(this.options.output);
            if (!$output.length)
            {
                console.error('Missing output field.');
                return;
            }
            this.$output = $output;

            var self = this;
            this.$target.find(':input').each(function ()
            {
                var $input = $(this);
                if ($input.is('[data-option-name]') || $input.is('[data-bb-value]'))
                {
                    $input.on('keypress change paste input', XF.proxy(self, 'generateOutput'));
                }
            });

            this.$target.find('[data-xf-click="sv-advanced-bb-code-pack--preview-click"]').on('click', XF.proxy(this, 'generateOutput'));
            this.$target.on('submit', XF.proxy(this, 'generateOutput'));
        },

        generateOutput: function (e)
        {
            var self = this,
                options = [],
                values = [],
                inputsScanned = [],
                showErrors = e.type === 'submit';

            this.$target.find(':input:visible, input[type=hidden]').each(function ()
            {
                var $input = $(this),
                    inputName = $input.attr('name'),
                    id, label, text;

                if (!$input.is(self.options.output) && !$input.is('[type="submit"]'))
                {
                    if ($input.is('[data-option-name]'))
                    {
                        var optionName = $input.data('option-name'),
                            optionType = $input.data('option-type') || 'string',
                            nameless = $input.data('option-nameless') === 'true';

                        if ($.inArray(inputName, inputsScanned) === -1)
                        {
                            inputsScanned.push(inputName);

                            if (!optionName)
                            {
                                console.error('Invalid option provided.');
                                return;
                            }

                            var optionValue = self.valueFromInput(self.$target, $input, 'option');
                            if (optionValue)
                            {
                                options.push(self.optionStr(optionName, optionValue, optionType, $input, nameless));
                            }
                        }
                    }
                    else if (self.getFormattedValue('bool', $input.data('bb-value'), $input))
                    {
                        var parseValue = true;
                        if ($input.parent().is('[data-increment-format]') && $input.parent().is(':last-child') && $input.parent().parent().find('li').length > 1)
                        {
                            parseValue = false;
                        }

                        if (parseValue)
                        {
                            var isRequire = self.getFormattedValue('bool', $input.data('bb-require'), $input),
                                requireIfNot = $input.data('bb-require-if-not') || null,
                                requirePhrase = self.getFormattedValue('str', $input.data('bb-require-phrase'), $input) || null,
                                valueType = $input.data('bb-type') || 'string',
                                value = self.valueFromInput(self.$target, $input, 'bb_code');

                            if (!value && showErrors)
                            {
                                if (isRequire)
                                {
                                    id = $input.attr('id');
                                    if (id) {
                                        label = $("label[for='" + id + "']");
                                        if (label.length === 1) {
                                            text = label.text().trim();
                                            if (text) {
                                                inputName = text;
                                            }
                                        }
                                    }

                                    var showErrorForCurrentInput = true,
                                        otherInputName = '';
                                    if (typeof requireIfNot === 'string')
                                    {
                                        var otherElement = self.$target.find(requireIfNot);
                                        if (otherElement.length)
                                        {
                                            showErrorForCurrentInput = !self.getFormattedValue(self.valueFromInput(self.$target, $input, 'bb_code'));
                                            if (showErrorForCurrentInput)
                                            {
                                                otherInputName = otherElement.attr('name');
                                                id = otherElement.attr('id');
                                                if (id) {
                                                    label = $("label[for='" + id + "']");
                                                    if (label.length === 1) {
                                                        text = label.text().trim();
                                                        if (text) {
                                                            otherInputName = text;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    if (showErrorForCurrentInput)
                                    {
                                        if (!requirePhrase)
                                        {
                                            requirePhrase = 'please_enter_value_for_required_field_x';
                                        }

                                        XF.overlayMessage(XF.phrase('oops_we_ran_into_some_problems'), XF.phrase(requirePhrase, {
                                            '{field}': inputName,
                                            '{otherField}': otherInputName
                                        }));

                                        e.stopImmediatePropagation();
                                        e.preventDefault();

                                        return;
                                    }
                                }
                            }

                            value = self.valueStr(value, valueType, $input);
                            if (value)
                            {
                                values.push(value);
                            }
                        }
                    }
                }
            });

            var tag = XF.htmlspecialchars(this.options.bbCode).toString().toUpperCase();
            var bbCode = '',
                bbCodeStart = '',
                bbCodeEnd = '',
                respectPreviousCursor = false;
            if (options.length || values.length)
            {
                if (options.length)
                {
                    options = $.trim(options.join(this.options.oldFormat ? '|' : ' '));
                    if (options)
                    {
                        var optionPrefix = ' ';
                        if (this.options.oldFormat || this.options.singleOption)
                        {
                            optionPrefix = '=';
                        }
                        options = optionPrefix + options;
                    }
                }

                if (options || values.length)
                {
                    var padding = this.options.newLines ? '<br/>\n' : '';
                    respectPreviousCursor = values.length === 0;
                    bbCodeStart = '[' + tag + options + ']' + padding;
                    bbCodeEnd = padding + '[/' + tag + ']';

                    if (values.length)
                    {
                        values = values.join(this.options.newLines ? '<br/>\n' : ' ');
                        var caretPlaceHolderTpl = '<span data-caret-placeholder="true"></span>',
                            re = new RegExp(caretPlaceHolderTpl, "gi"),
                            hasCaretPlaceholder = (values.match(re) || []).length !== 0;

                        if (hasCaretPlaceholder)
                        {
                            respectPreviousCursor = true;
                            var tmpPlaceholder = '<span id="' + Math.random().toString(36).substr(10, 20) + '"></span>';
                            var split = values
                                .replace(caretPlaceHolderTpl, tmpPlaceholder)
                                .replace(re, '')
                                .split(tmpPlaceholder);
                            values = '';
                            bbCodeStart = bbCodeStart + split[0];
                            bbCodeEnd = split[1] + bbCodeEnd;
                        }
                    }
                    else
                    {
                        values = padding;
                    }

                    bbCode = bbCodeStart + values + bbCodeEnd
                }
            }
            else
            {
                respectPreviousCursor = true;
                bbCodeStart = '[' + tag + ']<br/>\n';
                bbCodeEnd = '<br/>\n[/' + tag + ']';
                bbCode = bbCodeStart + bbCodeEnd;
            }

            this.$output.data('bb-code-start', bbCodeStart);
            this.$output.data('bb-code-end', bbCodeEnd);
            this.$output.data('bb-code-respect-cursor', respectPreviousCursor ? 'true' : 'false');
            this.$output.val(bbCode);
        },

        valueFromInput: function (form, input, inputType)
        {
            if (!inputType)
            {
                console.error('No input type provided.');
                return;
            }

            var self = this,
                inputName = input.attr('name'),
                valueType = null,
                delimiter = null;

            if (inputType === 'option')
            {
                valueType = input.data('option-type');
                delimiter = input.data('option-delimiter');
            }
            else if (inputType === 'bb_code')
            {
                valueType = input.data('bb-type');
                delimiter = input.data('bb-delimiter');
            }

            valueType = valueType || 'string';
            delimiter = delimiter || ',';

            if (valueType === 'time')
            {
                var name = input.attr('name');
                // assemble the parts
                var ymd = form.find('[name="' + name + '[ymd]' + '"]').val().trim();
                if (ymd === '') {
                    // todo fix-me
                    input.val('');
                    return '';
                }
                var zeroPad = function (n) {
                    return n.length >= 2 ? n : new Array(2 - n.length + 1).join('0') + n;
                };
                var hh = form.find('[name="' + name + '[hh]' + '"]').val();
                var mm = form.find('[name="' + name + '[mm]' + '"]').val();
                var ss = form.find('[name="' + name + '[ss]' + '"]').val();
                var tz = form.find('[name="' + name + '[tz]' + '"]').val();

                input.val(ymd + ' ' + zeroPad(hh)+ ':' + zeroPad(mm) + ':' + zeroPad(ss) + ' ' + tz);

                return input.val();
            }

            if (input.is('[type="text"]') || input.is('textarea') || input.is('[type="number"]'))
            {
                return input.val();
            }
            else if (input.is('[type="radio"]'))
            {
                var checkedRadio = form.find('[name="' + inputName + '"]:checked');
                if (checkedRadio.length === 1)
                {
                    return checkedRadio.val();
                }
            }
            else if (input.is('[type="checkbox"]'))
            {
                var checkedBoxes = form.find('[name="' + inputName + '"]:checked');
                if (checkedBoxes.length)
                {
                    var checkedValues = [];
                    checkedBoxes.each(function ()
                    {
                        var checkbox = $(this);
                        checkedValues.push(self.getFormattedValue(valueType, checkbox.val()));
                    });

                    return checkedValues.join(delimiter);
                }
            }
            else if (input.is('select'))
            {
                var isMultiple = input.prop('multiple');
                if (isMultiple)
                {
                    var selectedValues = [];
                    $input.find('option:selected').each(function ()
                    {
                        var radio = $(this);
                        selectedValues.push(self.getFormattedValue(valueType, radio.val()));
                    });

                    return selectedValues.join(delimiter);
                }
                else
                {
                    return input.val();
                }
            }

            return null;
        },

        getFormattedValue: function (type, value, input, trusted)
        {
            trusted = trusted || false;
            value = $.trim(value);
            var originalType = typeof value;
            var inputName = null;
            if (input)
            {
                inputName = input.attr('name');
            }

            switch (type)
            {
                case 'time':
                case 'str':
                case 'string':
                    if (input.attr('data-value') === value)
                    {
                        return null;
                    }
                    if (originalType !== 'string')
                    {
                        value = String(value);
                    }
                    break;

                case 'int':
                case 'number':
                case 'seconds':
                    if (input.attr('data-value') === value)
                    {
                        return null;
                    }
                    else if (originalType !== 'number')
                    {
                        value = Number(value);
                        if (isNaN(value))
                        {
                            value = 0;
                        }
                        if (type === 'seconds')
                        {
                            value = value + 's';
                        }
                    }
                    break;

                case 'bool':
                case 'boolean':
                    if (originalType !== 'boolean')
                    {
                        switch (value)
                        {
                            case 'true':
                            case 'yes':
                            case 'on':
                            case '1':
                                value = true;
                                break;

                            default:
                                value = false;
                        }
                        trusted = true;
                    }
                    break;

                case 'width':
                case 'height':
                    if (input)
                    {
                        var unitInput = input.parent().find('[name="' + inputName + '_unit"]');
                        if (!unitInput.length)
                        {
                            console.error('No unit input available for ' + type + ' input');
                            return;
                        }

                        var unit = unitInput.val();
                        switch (unit)
                        {
                            case 'pixels':
                                unit = 'px';
                                break;

                            case 'percentage':
                                unit = '%';
                                break;

                            case '':
                            case 'auto':
                                unit = null;
                        }

                        if (unit)
                        {
                            value = this.getFormattedValue('int', value, input).toString();
                            if (value)
                            {
                                value += unit;
                            }
                        }
                        else
                        {
                            value = null;
                        }
                        trusted = true;
                    }
                    break;

                case 'slide':
                case 'slide_header':
                    var tag = input.data('bb-type').toUpperCase();
                    var body = value ? this.getFormattedValue('str', value, input) : '';
                    var braceType = input.data('brace-type');
                    var openBrace = braceType === 'new' ? '[' : '{';
                    var closeBrace = braceType ? ']' : '}';
                    if (tag === 'SLIDE') {
                        body += '<br/><span data-caret-placeholder="true"></span><br/>\n';
                    }

                    value = openBrace + tag + closeBrace + body + openBrace + '/' + tag + closeBrace;
                    trusted = true;
                    break;
            }

            if (!trusted)
            {
                value = XF.htmlspecialchars(value);
            }

            return value;
        },

        optionStr: function (name, value, type, input, nameless, trusted)
        {
            type = type || 'string';
            trusted = trusted || false;
            value = this.getFormattedValue(type, value, input, trusted);

            if (!value)
            {
                return '';
            }

            name = XF.htmlspecialchars(this.options.oldFormat ? name : name.replace('-', '_'));
            nameless = nameless || false;

            if (this.options.oldFormat || this.options.singleOption)
            {
                if (nameless || this.options.singleOption)
                {
                    return '"' + value + '"';
                }

                return name + ':' + value;
            }

            return name + '="' + value + '"';
        },

        valueStr: function (value, type, input, trusted)
        {
            type = type || 'string';
            trusted = trusted || false;
            value = this.getFormattedValue(type, value, input, trusted);

            if (!value)
            {
                return '';
            }

            return value;
        }
    });

    XF.Element.register('svBbCode-dialog-form', 'SV.AdvancedBBCode.DialogForm');
})(jQuery, window, document);
