var SV = window.SV || {};
SV.AdvancedBBCode = SV.AdvancedBBCode || {};
SV.WordCounter = SV.WordCounter || {};

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined)
{
    "use strict";

    SV.AdvancedBBCode.editorConfig = {
        dialog: [],
        tags: {
            abbr: false,
            h4: false,
        },
        shimH1: false,
        shimHr: false,
        pasteQuotesAsBbCode: false,
        normalizeQuoteChars: false,
        extraBrSanitization: 0,
        hrStyle: 0,
        disableBbCodeModeClicks: 0,
        disableDisableBbCodeModeClicks: 0,
        disableBbCodeModeClicksPhrase: '',
    };

    XF.Editor = XF.extend(XF.Editor, {
        __backup: {
            'normalizePaste': 'svAdvancedBbCode_normalizePaste',
            'editorInit': 'svAdvancedBbCode_editorInit',
        },

        editorInit: function() {

            // Workaround for Froala not updating on resize
            // See https://github.com/froala/react-froala-wysiwyg/issues/234#issuecomment-608976725

            this.svAdvancedBbCode_editorInit();

            this.ed.events.on('commands.before', function(cmd)
            {
                if (cmd === 'fullscreen') {
                    setTimeout(() => window.dispatchEvent(new Event('resize')), 0);
                }
            });
        },

        normalizePaste: function(content) {
            var $fragWrapper = $('<div />').html(content),
                rewriteTags = [];

            // restore styling injected by this add-on
            // froala ignores attributes that match `data-fr-*`
            $fragWrapper.find('[data-fr-sv-class]').each(function() {
                var $self = $(this);
                $self.addClass($self.attr('data-fr-sv-class'));
                $self.removeAttr('data-fr-sv-class');
            });

            if (SV.AdvancedBBCode.editorConfig.tags.abbr) {
                $fragWrapper.find('abbr').replaceWith(function () {
                    var title = '' + $(this).attr('title');
                    return '[ABBR' + (title.length ? '="' + title + '"' : '') + ']' + this.innerHTML + "[/ABBR]";
                });
            }

            if (SV.AdvancedBBCode.editorConfig.tags.shimH1) {
                // add default styling for <hr> tags
                var hrStyle = SV.AdvancedBBCode.editorConfig.hrStyle;
                $fragWrapper.find('hr:not(.bbc-hr)')
                    .addClass('bbc-hr' + (hrStyle ? ' bbc-hr--' + hrStyle : ''));

                var tagsToRewrite = $fragWrapper.find('sub, sup, h1, h2, h3, h4');
                if (tagsToRewrite.length !== 0) {
                    rewriteTags = ['SVBBCODE_SUB', 'SVBBCODE_SUP', 'SVBBCODE_H1', 'SVBBCODE_H2', 'SVBBCODE_H3', 'SVBBCODE_H4'];
                    tagsToRewrite.replaceWith(function () {
                        var $replacement = $('<SVBBCODE_' + this.tagName + ' />');
                        $replacement.html(this.innerHTML);
                        $replacement.addClass(this.className);
                        return $replacement;
                    });
                }
            }

            content = $fragWrapper.html();

            content = this.svAdvancedBbCode_normalizePaste(content);

            if (rewriteTags.length !== 0 || SV.AdvancedBBCode.editorConfig.extraBrSanitization) {
                $fragWrapper = $('<div />').html(content);

                if (SV.AdvancedBBCode.editorConfig.extraBrSanitization) {
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.normalizeBrForEditor($fragWrapper);
                }

                if (rewriteTags.length !== 0) {
                    var rewriteTagFilter = rewriteTags.join(', ');
                    $fragWrapper.find(rewriteTagFilter).replaceWith(function () {
                        var tag = this.tagName.replace('SVBBCODE_', '');
                        var $replacement = $('<' + tag + ' />');
                        $replacement.html(this.innerHTML);
                        $replacement.addClass(this.className);

                        switch (tag) {
                            case 'H1':
                            case 'H2':
                            case 'H3':
                            case 'H4':
                                if (!$replacement.hasClass('bbc-header')) {
                                    // add default styling for h1/h2/h3/h4 tags
                                    $replacement.addClass('bbc-header');
                                }
                                break;
                        }
                        return $replacement;
                    });
                }

                content = $fragWrapper.html();
            }

            return content;
        }
    });


    SV.AdvancedBBCode.EditorButtons = {

        config: function (event, config, xfEditor)
        {
            var tags = SV.AdvancedBBCode.editorConfig.tags || {};
            if (tags.abbr) {
                config.htmlAllowedTags.push('abbr');
            }
            if (SV.AdvancedBBCode.editorConfig.shimH1) {
                config.paragraphFormat = {
                    N: 'Normal',
                    H1: 'Heading 1',
                    H2: 'Heading 2',
                    H3: 'Heading 3'
                };
                if (tags.h4) {
                    config.paragraphFormat['H4'] = 'Heading 4';
                }
            }
            if (typeof $.FE.DEFAULTS.wordCounter !== 'undefined') {
                config.wordCounter = true;
                config.wordCounterBbCode = true;
                config.wordCounterLabel = XF.phrase('sv_bbcode_word_count');

                if ('svBbCodeWordCountPhrases' in XF.config)
                {
                    config.wordCounterLabels = XF.config.svBbCodeWordCountPhrases;
                }

                config.wordCounterTimeout = SV.WordCounter.timeoutInterval || 0;
                if (config.wordCounterTimeout < 0) {
                    config.wordCounterTimeout = 600;
                }

                if (config.pluginsEnabled)
                {
                    config.pluginsEnabled.push('wordCounter');
                }
            }

/*
            config.pasteAllowedStyleProps = config.pasteAllowedStyleProps || [];
            [
                //'color','font-family',
                //'font-size','text-indent','text-align'
            ].forEach(function(element) { config.pasteAllowedStyleProps.push(element); });
            if ('pasteDeniedAttrs' in config) {
                for (var i = 0; i < config.pasteDeniedAttrs.length; i++) {
                    if (config.pasteDeniedAttrs[i] === 'style') {
                        config.pasteDeniedAttrs.splice(i, 1);
                        break;
                    }
                }
            } else {
                config.pasteDeniedAttrs = ['id','class'];
            }
*/
        },

        normalizeBrForEditor: function (content) {
            if ('normalizeBrForEditor' in XF.EditorHelpers){
                return XF.EditorHelpers.normalizeBrForEditor(content);
            }

            var asString = typeof content === 'string',
                $fragWrapper;
            if (asString) {
                $fragWrapper = $('<div />').html(content);
            } else {
                $fragWrapper = content;
            }
            var checkNodeMatch = function ($node, elementType) {
                var node = $node.get(0);
                return $node.is(elementType) &&
                    node.className === '' &&
                    !node.hasAttribute('id') &&
                    !node.hasAttribute('style')
            };

            // Workaround editor behaviour that a <br> should not be the first or last child of a <p> tag
            // <p><br>...</p>; editor can delete too many lines
            // <p>...<br></p>; editor can delete too few lines
            // <p>word1 word2<br></p>; editor will delete the space character between word1/word2 when backspacing word2 out
            // Fixed in XF2.2.7

            $fragWrapper.children('p').each(function () {
                if (this.childNodes.length !== 1) {
                    return;
                }
                var $firstChild = $(this.childNodes[0]);
                if (checkNodeMatch($firstChild, 'span')) {
                    $(this).html($firstChild.html());
                }
            });
            $fragWrapper.children('p').each(function() {
                if (this.childNodes.length <= 1) {
                    return;
                }
                var $firstChild = $(this.childNodes[0]);
                if (checkNodeMatch($firstChild, 'br')) {
                    $(this).before($('<p>').append($firstChild));
                }
            });
            $fragWrapper.children('p').each(function() {
                if (this.childNodes.length <= 1) {
                    return;
                }
                var $lastChild = $(this.childNodes[this.childNodes.length - 1]);
                if (checkNodeMatch($lastChild, 'br')) {
                    $lastChild.remove();
                }
            });

            return asString ? $fragWrapper.html() : $fragWrapper;
        },

        setupFragWrapper: function (content, normalizeQuoteCharacters, pruneMetadata, inlineClassToCss) {

            if (normalizeQuoteCharacters) {
                content = SV.AdvancedBBCode.EditorButtons.cleanupQuoteCharactersCopyPaste(content);
            }
            // noinspection UnnecessaryLocalVariableJS
            var frag = $.parseHTML(content),
                $fragWrapper = $('<div />').html(frag);

            if (inlineClassToCss) {
                var allowedAttributes = typeof inlineClassToCss === 'object';
                var html = '',
                    iframe = $('<iframe id="ClassToInlineStyleImporter" style="position: absolute;width:0;height:0;border:0;"></iframe>');
                try {
                    iframe.html($fragWrapper.html());
                    iframe.appendTo($('body'));

                    // extract computed set in one go as getComputedStyle is dynamic
                    var list = [];
                    $.uniqueSort(iframe.find('p, span, hr, [class]')).each(function (index, element) {
                        var css = {},
                            computedStyle = window.getComputedStyle(element);
                        if (computedStyle.length === 0) {
                            return;
                        }
                        for (var i = computedStyle.length; i--;) {
                            var nameString = computedStyle[i];
                            if (allowedAttributes && !(nameString in inlineClassToCss)) {
                                continue;
                            }
                            var val = computedStyle.getPropertyValue(nameString);
                            if (val !== '') {
                                css[nameString] = val;
                            }
                        }

                        if (css.length !== 0) {
                            list.push([$(element), css]);
                        }
                    });
                    // apply once we have a static list
                    for(var i=0;i<list.length;i++) {
                        var $e = list[i][0],
                            css = list[i][1];

                        var existingInlineStyle = $e.attr('style')
                        $e.css(css);
                        if (existingInlineStyle) {
                            $e.attr('style', $e.attr('style') + ';' + existingInlineStyle)
                        }
                    }

                    html = iframe.html();
                }
                catch(exception) {
                    console.log(exception);
                }
                iframe.remove();
                if (html !== '') {
                    $fragWrapper = $('<div />').html(html);
                }
            }

            if (pruneMetadata) {
                $fragWrapper.contents().filter(function () {
                    return this.nodeType === Node.COMMENT_NODE;
                }).remove();
                $fragWrapper.find('> xml, > meta, > style, > link, > title').remove();

                $fragWrapper = SV.AdvancedBBCode.EditorButtons.setupFragWrapper($fragWrapper.html().trim());
            }

            return $fragWrapper;
        },

        cleanupQuoteCharactersCopyPaste: function (content) {
            return content.replace(/&#x2018;|&#x2019;|[\u2018\u2019]/g, "'").replace(/&#x201C;|&#x201D;|[\u201C\u201D]/g, '"');
        },

        cleanupWordNewlinesCopyPaste: function ($fragWrapper) {
            // prune span[lanq] to simplify processing. See from older MS Word
            $fragWrapper.find('span[lang], span[style*="mso-ansi-language"], span[style*="mso-fareast-language"]').each(function(index, element) {
                var $self = $(element);
                $self.replaceWith($self.html());
            });

            $fragWrapper.find('div hr').each(function(index, element) {
                var $self = $(element);
                $self.unwrap();
            });

            // MsoNoSpacing ==  "No spacing" paragraph style
            // MsoNormal ==  Normal paragraph style, extra line after each paragraph
            // However, this is derived from the line-height via the css which can sometimes change things.
            $fragWrapper.children('p.MsoNoSpacing,p.MsoNormal').each(function(index, element) {
                var $self = $(element),
                    $e = $('<span />');
                $e.attr('style', $self.attr('style'));
                $self.contents().each(function() {
                    if(this.nodeType === Node.COMMENT_NODE || this.tagName === 'O:P') {
                        $(this).remove();
                    }
                });
                $e.html($self.html());
                $e.addClass(element.className);

                SV.AdvancedBBCode.EditorButtons.appendBrForSpanRewrite($e, 0);

                var marginBottomAttr = '' + element.style.marginBottom;
                var extraAppendOnEmpty;
                if (marginBottomAttr !== '') {
                    var bottomPadding = SV.AdvancedBBCode.EditorButtons.getAttrLineHeightPercent(element, marginBottomAttr, 0);
                    if (bottomPadding > 0.1) {
                        element.style.marginBottom = '';
                        element.style.marginTop = '';
                        extraAppendOnEmpty = 1;
                    } else {
                        extraAppendOnEmpty = 0;
                    }
                } else {
                    extraAppendOnEmpty = $e.hasClass('MsoNormal');
                }

                if (extraAppendOnEmpty && $e.length > 0) {
                    $e.append($('<br>'));
                }

                $self.replaceWith($e);
            });

            return $fragWrapper;
        },

        appendBrForSpanRewrite($e, extraAppendOnEmpty = 0, appendDefault = 1) {
            // append a newline if one isn't in the last child of the paragraph (skipping whitespace)
            var
                append  = appendDefault,
                childNodes = $e.get(0).childNodes;
            for(var i = childNodes.length -1; i >= 0; i--) {
                var node = childNodes[i];
                if (node.nodeType === Node.TEXT_NODE && (!node.nodeValue || /^[\s\n]+$/.test(node.nodeValue))) {
                    continue;
                }

                append = node.tagName !== 'BR';
                break;
            }
            if (append) {
                if (extraAppendOnEmpty && childNodes.length > 0) {
                    $e.append($('<br>'));
                }
                $e.append($('<br>'));
            }
        },

        getAttrLineHeightPercent:function(element, attr, defaultVal) {
            // if line-height is in px, check the font-size to compute the percentage line-height
            // otherwise default to 100%
            // note; not all paragraph styles add an explicit line-height, and
            // projecting the default style into inline attributes returns px values

            attr = attr ? attr : defaultVal + '%';

            var lineHeight = defaultVal,
                match = attr.match(/^(\d+(?:\.\d+)?)px$/)
            if (match && match[1]) {
                lineHeight = parseFloat(match[1]);
                attr = '' + element.style.fontSize;
                match = attr.match(/^(\d+)px$/)
                if (match && match[1]) {
                    var fontSize = parseFloat(match[1]);
                    lineHeight = ((100 * lineHeight) / fontSize) | 0;
                }
            } else {
                match = attr.match(/^(\d+)\%$/);
                if (match && match[1]) {
                    lineHeight = match[1] | 0;
                }
            }

            return lineHeight;
        },

        cleanupLibreOfficeNewlinesCopyPaste: function ($fragWrapper, lineHeightThreshold) {

            // OpenOffice supports inserting an image as a horizontal ruler, which LibreOffice supports
            $fragWrapper.find('p > img[src*="gallery/rulers"]').each(function(index, element) {
                var $self = $(element);
                if (/file:\/\/.*\/gallery\/rulers\/.*.gif/i.test($self.attr('src'))) {

                    var $e = $('<hr />');
                    $e.attr('style', $self.attr('style'));
                    $self.replaceWith($e);
                }
            });

            // a <p> with a line-height of >= 1 is effectively a span with a bunch of <br> on the end
            // except if it is <p><br><p> which is a blank line
            $fragWrapper.children('p').each(function(index, element) {
                var lineHeight = SV.AdvancedBBCode.EditorButtons.getAttrLineHeightPercent(element, element.style.lineHeight, 100);
                if (lineHeight) {
                    var $self = $(element),
                        $e = $('<span />');

                    element.style.lineHeight = '';
                    $e.attr('style', $self.attr('style'));
                    $e.html($self.html());

                    SV.AdvancedBBCode.EditorButtons.appendBrForSpanRewrite($e, 0);

                    // "Text Body" paragraph formatting threshold, varies between OpenOffice and LibreOffice
                    if (lineHeight >= lineHeightThreshold && lineHeight < 200) {
                        lineHeight = 200;
                    }
                    var lineHeightPercentage = Math.floor(lineHeight / 100) | 0;
                    while (lineHeightPercentage > 1) {
                        lineHeightPercentage--;
                        $e.append($('<br>'));
                    }


                    $self.replaceWith($e);
                }
            });

            $fragWrapper.children('hr').each(function(index, element) {
                var lineHeight = SV.AdvancedBBCode.EditorButtons.getAttrLineHeightPercent(element, element.style.lineHeight, 100);
                if (lineHeight) {
                    var $self = $(element);

                    // round up if it isn't small enough
                    if (lineHeight >= 115 && lineHeight < 200) {
                        lineHeight = 200;
                    }

                    var lineHeightPercentage = Math.floor(lineHeight / 100) | 0;
                    while (lineHeightPercentage > 1) {
                        lineHeightPercentage--;
                        $self.after($('<br>'));
                    }
                }
            });

            return $fragWrapper;
        },

        cleanupOpenOfficeNewlinesCopyPaste: function ($fragWrapper) {

            // cleanupLibreOfficeNewlinesCopyPaste rewrites p => span, and handles newlines

            // Increase spacing; margin-top/margin-bottom - 0.1cm - 0.31cm - 1 line
            // Text Body paragraph formatting; margin-bottom:0.21cm - 1 line
            // projecting the default style into inline attributes returns px values
            $fragWrapper.children('span[style*="margin"]').each(function(index, element) {
                var marginBottomAttr = '' + element.style.marginBottom;
                if (marginBottomAttr === '') {
                    return;
                }
                var lineHeight = SV.AdvancedBBCode.EditorButtons.getAttrLineHeightPercent(element, marginBottomAttr, 0);
                if (lineHeight >= 115 / 2) {
                    element.style.marginBottom = '';
                    element.style.marginTop = '';
                    $(element).append($('<br>'));
                }
            });

            return $fragWrapper;
        },

        cleanupHtmlCopyPaste: function ($fragWrapper, detectLineEditor, isGoogleDocs)
        {
            var nodes;

            // handle discord code blocks
            nodes = $fragWrapper.find('pre > code.hljs');
            nodes.each(function(index, element){
                var $self = $(element),
                    $parent = $self.parent(),
                    $inner = $('<span />');

                // convert to a [code=XX][/code] block
                var match = $self.attr("class").match(/.*\s+hljs\s+([^\s]+)$/i);
                var type = match ? "=" + match[1] : '';

                $inner.attr('style', $self.attr('style') + ';' + $parent.attr('style'));
                $inner.text($self.text());

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[code'+type+']\n');
                $outer.append($inner);
                $outer.append('[/code]');

                $parent.replaceWith($outer);
            });
            // handle discord/XF inline code blocks
            nodes = $fragWrapper.find('code.inline, code.bbCodeInline');
            nodes.each(function(index, element){
                var $self = $(element);

                // convert to a [icode][/icode] block
                var $inner = $("<span />");
                $inner.attr('style', $self.attr('style'));
                $inner.text($self.text());

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[icode]');
                $outer.append($inner);
                $outer.append('[/icode]');

                $self.replaceWith($outer);
            });
            // handle discord inline spoiler blocks
            nodes = $fragWrapper.find('span[class*="spoilerText"][aria-expanded] > span[aria-hidden], span.bbCodeInlineSpoiler, span.bbc-spoiler');
            nodes.each(function(index, element){
                var $self = $(element);

                $self.removeClass('bbc-spoiler bbc-spoiler-instant');
                // wrap with [ispoiler]...[/ispoiler] block, but leave the original div alone to preserve formatting
                $self.prepend("[ispoiler]");
                $self.append("[/ispoiler]");
            });

            // convert XF usernames into actual @username text
            nodes = $.uniqueSort($fragWrapper.find('.message-name .username[itemprop="name"], .username[itemprop="name"]'))
            nodes.each(function(index, element){
                var $self = $(element),
                    $parent = $self.parent(),
                    text = $self.text(),
                    $e = $('<span />');

                text = text.replace(/^@/, '');
                var userId = $self.data('user-id') | 0;
                var prefix = userId ? '@' : '';
                text = prefix + text + ' ';

                $e.text(text);

                if ($parent.hasClass('message-name')) {
                    $parent.replaceWith($e);
                } else {
                    $self.replaceWith($e);
                }
            });

            nodes = $fragWrapper.find('h1, h2, h3, h4, h5, h6')
            nodes.each(function(index, element){
                var $self = $(element),
                    tagName = element.tagName;
                if (tagName === 'H5' || tagName === 'H6') {
                    tagName = 'H4';
                }

                // convert header to bb-code, as editor behaviour is bad
                var $inner = $('<span />');
                $inner.attr('style', $self.attr('style'));
                // due to wonky XF editor behaviour trim leading newlines
                $inner.html($self.html().replace(/^\n+/, ''));

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[' + tagName + ']');
                $outer.append($inner);
                $outer.append('[/' + tagName + ']');

                $self.replaceWith($outer);
            });

            if (SV.AdvancedBBCode.editorConfig.pasteQuotesAsBbCode) {
                nodes = $fragWrapper.find('blockquote');
                nodes.each(function (index, element) {
                    var $self = $(element),
                        $inner = $("<span class='blockquote-span' />");

                    $inner.attr('style', $self.attr('style'));
                    $inner.html($self.html());

                    var $outer = $('<p data-xf-p="1" />');
                    $outer.append('[QUOTE]\n');
                    $outer.append($inner);
                    $outer.append('[/QUOTE]');

                    $self.replaceWith($outer);
                });
            }

            // convert apple-spaces <span className="Apple-converted-space">&nbsp;</span>
            nodes = $fragWrapper.find('span.Apple-converted-space');
            nodes.each(function(index, element) {
                var $self = $(element),
                    html = $self.html();
                if (html === '&nbsp;') {
                    $self.replaceWith(' ');
                }
            });

            nodes = $fragWrapper.find('p, span, div, blockquote, code');
            // possible google docs paste. will be a list of <p> with a collection of <span> or a collection of <span>
            nodes.each(function(index, element){
                var $self = $(element),
                    wrap = [],
                    attr,
                    bbCode;

                // font-weight: >700 => <b>
                attr = element.style.fontWeight;
                element.style.fontWeight = "";
                if (Number(attr) >= 700 || attr === 'bold') {
                    wrap.push('strong');
                }

                if (!$self.parent('pre').length) {
                    attr = element.style.whiteSpace;
                    if (attr === '' && ($self.is('blockquote') || $self.hasClass('blockquote-span'))) {
                        // fetch from parent
                        attr = $self.css('white-space');
                        if (attr === '') {
                            var $parents = $self.parent('[style*="white-space:"]').first();
                            if ($parents.length) {
                                attr = $parents.get(0).style.whiteSpace;
                            }
                        }
                    }
                    if (/^pre(-.*)?$|^break-spaces$/i.test(attr)) {
                        element.style.whiteSpace = '';
                        var patchTextFragment = function(node) {
                            if (!node.hasChildNodes()) {
                                return;
                            }
                            var children = node.childNodes;
                            // node list is "live" so, need to bump i based on adding/removing nodes!
                            for (let i = 0; i < children.length; i++) {
                                var childNode = children[i];

                                if (childNode.hasChildNodes()) {
                                    patchTextFragment(childNode);
                                } else if (childNode.nodeType === Node.TEXT_NODE && childNode.nodeValue) {
                                    var textFragments = childNode.nodeValue.split(/\r?\n/)
                                    if (textFragments.length > 1) {
                                        textFragments.forEach(function (val, i2) {
                                            var text = document.createTextNode(val);
                                            node.insertBefore(text, childNode);
                                            i++;
                                            if (i2 + 1 < textFragments.length) {
                                                var br = document.createElement("br")
                                                node.insertBefore(br, childNode);
                                                i++;
                                            }

                                        });
                                        node.removeChild(childNode);
                                        i--;
                                    }
                                }
                            }
                        }
                        element.normalize();
                        patchTextFragment(element);
                    }
                }

                // font-style:italic (or oblique) => <em>
                attr = element.style.fontStyle;
                element.style.fontStyle = "";
                if (/^italic|^oblique/i.test(attr)) {
                    wrap.push('em');
                }

                // text-decoration: line-through/underline => <s>/<u>
                attr = element.style.textDecoration;
                element.style.textDecoration = "";
                if (attr === 'line-through') {
                    wrap.push('s');
                } else if (attr === 'underline') {
                    wrap.push('u');
                }

                // vertical-align: super/syb => <sup>/<sub>
                attr = element.style.verticalAlign;
                element.style.verticalAlign = "";
                if (attr === 'super') {
                    wrap.push('sup');
                } else if (attr === 'sub') {
                    wrap.push('sub');
                }

                // Editor doesn't preserve 'text-align' so rewrite into bb-code
                // text-align: center/justify/end => [centre]/[justify]/[right] bb-code
                attr = element.style.textAlign;
                element.style.textAlign = "";
                bbCode = '';
                switch(attr) {
                    case 'center':
                        bbCode = 'center';
                        break;
//                    case 'justify':
//                    case 'justify-all':
//                        bbCode = 'justify';
//                        break;
                    case 'end':
                    case 'right':
                        bbCode = 'right';
                        break;
                }
                if (bbCode !== '') {
                    $self.prepend('<span>[' +  bbCode + ']</span>');
                    $self.append('<span>[/' +  bbCode + ']</span>');
                }


                // 'color:.*' => <color>
                // todo...

                if (wrap.length) {
                    var wrapper = $self;
                    $.each(wrap, function(k, v){
                        var attr, e;
                        if (Array.isArray(v)) {
                            e = v[0];
                            attr = v[1];
                        } else {
                            e = v;
                            attr = "";
                        }
                        wrapper = wrapper.wrapInner("<" + e + " "+ attr +  "></" + e + ">");
                    });
                }

                //element.style = {};
            });

            // froala ignores attributes that match `data-fr-*`
            // stash classes starting with "bbc-" into an attribute
            $fragWrapper.find('[data-fr-sv-class]').removeAttr('data-fr-sv-class');
            $fragWrapper.find('[class*="bbc-"]').each(function(){
                var results = this.className.match(/(bbc-[^\s]+)/gi);
                if (results.length !== 0) {
                    $(this).attr('data-fr-sv-class', results.join(' '));
                }
            });

            if (isGoogleDocs) {
                // Google Docs can output <p style="margin-top:1pt,margin-bottom:1pt"></p><br/> which results in unexpected doublespacing
                $fragWrapper.find('p[style*="margin-top:1pt"]').each(function () {
                    if (this.style.marginTop === '1pt' && this.style.marginBottom === '1pt' &&
                        (!this.nextElementSibling || this.nextElementSibling.tagName === 'BR')) {
                        this.style.marginTop = 0;
                        this.style.marginBottom = 0;
                    }
                });
            }

            if (!detectLineEditor) {
                return $fragWrapper;
            }

            // Detect "<p>...</p><p><br></p><p>...</p>" line-based editing and prevent XF injecting newlines
            // only p children, and at least one is just <p><br><p>, no p nodes which are empty
            var parentNode = $fragWrapper.get(0),
                childNodes = parentNode.childNodes;
            if (childNodes.length > 1) {
                var valid = true,
                    hasBr = false,
                    pruneList = [];
                for (var i = childNodes.length - 1; i >= 0; i--) {
                    var node = childNodes[i];
                    if (node.nodeType === Node.COMMENT_NODE) {
                        pruneList.push(node);
                        continue;
                    }
                    if (node.nodeType === Node.TEXT_NODE && (!node.nodeValue || /^[\s\n]+$/.test(node.nodeValue))) {
                        pruneList.push(node);
                        continue;
                    }
                    if (node.tagName === 'STYLE' ||
                        node.tagName === 'META' ||
                        node.tagName === 'LINK' ||
                        node.tagName === 'TITLE' ||
                        node.tagName === 'XML') {
                        pruneList.push(node);
                        continue;
                    }

                    var validChild,
                        childNodes2 = node.childNodes;
                    if (node.tagName !== 'P' && node.tagName !== 'HR') {
                        validChild = false;
                    } else {
                        validChild = false;
                        // check that this is node isn't empty, and flag if we have a br
                        for (var i2 = childNodes2.length - 1; i2 >= 0; i2--) {
                            var childNode = childNodes2[i2];
                            if (childNode.nodeType === Node.COMMENT_NODE) {
                                continue;
                            }
                            if (childNode.nodeType === Node.TEXT_NODE && (!childNode.nodeValue || /^[\s\n]+$/.test(childNode.nodeValue))) {
                                continue;
                            }

                            if (childNode.tagName === 'BR') {
                                hasBr = true;
                            }

                            // has some text or nodes
                            validChild = true;
                            break;
                        }
                    }

                    if (!validChild) {
                        valid = false;
                        break;
                    }
                }
                if (valid && hasBr) {
                    // `data-xf-p` tells XF not to double the <p>'s,
                    // however, need to remove non-block nodes between <p> tags or XF will wrap them in a <p> tag
                    for (i in pruneList) {
                        parentNode.removeChild(pruneList[i]);
                    }
                    $fragWrapper.children('p').attr('data-xf-p', '1');
                }
            }

            return $fragWrapper;
        },

        editorInit: function(event, ed, xfEditor)
        {
            ed.events.on('paste.after', function() {
                if (SV.AdvancedBBCode.editorConfig.extraBrSanitization) {
                    // handle inline copy-paste into a blank line, which results in <p>...<br></p>
                    SV.AdvancedBBCode.EditorButtons.normalizeBrForEditor(ed.$el);
                }
            });

            ed.events.on('paste.beforeCleanup', function(content)
            {
                // note; content is html in string form
                var $fragWrapper,
                    detectLineEditor = false,
                    isGoogleDocs = false,
                    normalizeQuoteChars = SV.AdvancedBBCode.editorConfig.normalizeQuoteChars;
                // XF cleanup occurs in 'paste.beforeCleanup', and word paste cleanup needs to occur before then
                // paste.wordPaste triggers to later to be used
                if (/class="?Mso|class='?Mso|class="?Xl|class='?Xl|class=Xl|style="[^"]*\bmso-|style='[^']*\bmso-|w:WordDocument/i.test(content)) {
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.setupFragWrapper(content, normalizeQuoteChars, true, {
                        // allowed style attributes to inline
                        'margin-bottom':1,
                        'margin-top':1
                    });
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.cleanupWordNewlinesCopyPaste($fragWrapper);
                } else if (/<meta[^>]+?LibreOffice/i.test(content)) {
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.setupFragWrapper(content, normalizeQuoteChars, true, true);
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.cleanupLibreOfficeNewlinesCopyPaste($fragWrapper, 115);
                } else if (/<meta[^>]+?OpenOffice/i.test(content)) {
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.setupFragWrapper(content, normalizeQuoteChars, true, true);
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.cleanupLibreOfficeNewlinesCopyPaste($fragWrapper, 141);
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.cleanupOpenOfficeNewlinesCopyPaste($fragWrapper);
                } else {
                    $fragWrapper = SV.AdvancedBBCode.EditorButtons.setupFragWrapper(content, normalizeQuoteChars, false);

                    // try detect google-docs
                    var $googleDocRoot = $fragWrapper.children('[id*="docs-internal-guid-"]');
                    isGoogleDocs = $googleDocRoot.length === 1 && $fragWrapper.children('p,hr').length === 0 && $googleDocRoot.children('p,hr,br').length > 0;
                    if (isGoogleDocs) {
                        $googleDocRoot.children().unwrap();
                    }

                    detectLineEditor = true;
                }

                $fragWrapper = SV.AdvancedBBCode.EditorButtons.cleanupHtmlCopyPaste($fragWrapper, detectLineEditor, isGoogleDocs);

                content = $fragWrapper.html();

                return content;
            }, true);

            var copyPasteContainerSelector = xfEditor.$target.data('debug-copy-paste-output'),
                $copyPasteContainer = copyPasteContainerSelector ? $(copyPasteContainerSelector) : null;
            if ($copyPasteContainer !== null && $copyPasteContainer.length !== 0) {
                var $copyNPasteDebugging = $('input[type="checkbox"][name="showCopyAndPasteDebugging"]');
                $copyNPasteDebugging.change(function() {
                    if (this.checked) {
                        $copyPasteContainer.show();
                    } else {
                        $copyPasteContainer.hide();
                    }
                });

                ed.events.on('paste.beforeCleanup', function(content)
                {
                    if (/^\s+$/i.test(content)) {
                        $copyPasteContainer.hide();
                        $copyPasteContainer.empty();
                    } else {
                        $copyPasteContainer.empty();
                        $copyPasteContainer.append('<hr>');
                        $copyPasteContainer.append($("<pre style='word-wrap:break-word;white-space:pre-wrap;' />").text(content));
                        $copyPasteContainer.append('<hr>');
                        if ($copyNPasteDebugging.length === 0 || $copyNPasteDebugging.get(0).checked) {
                            $copyPasteContainer.show();
                        }
                    }

                    return content;
                }, true);

                function checkContents() {
                    var text = ed.opts.wordCounterGetText
                        ? ed.opts.wordCounterGetText()
                        : (ed.el.innerText || "");
                    if (/^\s+$/i.test(text)) {
                        $copyPasteContainer.hide();
                        $copyPasteContainer.empty();
                    }
                }

                ed.events.on("contentChanged", checkContents);
            }
        },

        setupConfig: function($editorConfig) {
            try
            {
                SV.AdvancedBBCode.editorConfig = $.parseJSON($editorConfig.html()) || {};
            }
            catch (e)
            {
                console.error(e);
            }
        },

        setupLanguage: function()
        {
            var lang;

            try
            {
                lang = $.parseJSON($('.js-editorLanguage-ex').first().html()) || {};
            }
            catch (e)
            {
                console.error(e);
                lang = {};
            }

            Object.keys(lang).forEach(function (key) {
                $.FE.LANGUAGE['xf'][key] = lang[key];
            })
        },

        setupBlockTags: function(){
            // register our temp bb-codes as block-like tags
            $.FE.BLOCK_TAGS.push('svbbcode_h1');
            $.FE.BLOCK_TAGS.push('svbbcode_h2');
            $.FE.BLOCK_TAGS.push('svbbcode_h3');
            $.FE.BLOCK_TAGS.push('svbbcode_h4');
        },

        setupCommands: function() {
            if (!Array.isArray(SV.AdvancedBBCode.editorConfig.dialog)){
                return;
            }

            SV.AdvancedBBCode.editorConfig.dialog.forEach(function (element)
            {
                SV.AdvancedBBCode[element] = SV.AdvancedBBCode[element] || {};
                if (!SV.AdvancedBBCode[element].editorButton)
                {
                    SV.AdvancedBBCode[element].editorButton = {
                        init: function ()
                        {
                            SV.AdvancedBBCode[element].EditorDialog = XF.extend(XF.EditorDialog, {
                                simpleHtmlSnippet: null,
                                cache: false,
                                $container: null,

                                _init: function (overlay)
                                {
                                    this.simpleHtmlSnippet = overlay.$overlay.find('div#bb_code_output_simple_html').html();
                                    if (this.simpleHtmlSnippet && this.simpleHtmlSnippet.length) {
                                        this.cache = true;
                                        return;
                                    }

                                    overlay.$overlay.find('form').submit(XF.proxy(this, 'submit'));
                                },

                                beforeShow: function(overlay, e)
                                {
                                    if (this.simpleHtmlSnippet && this.simpleHtmlSnippet.length) {
                                        e.preventDefault();

                                        var self = this;
                                        setTimeout(function() {
                                            self.insertOutput(self.simpleHtmlSnippet);
                                        }, 0);

                                        return;
                                    }
                                    this.overlay = overlay;

                                    this._beforeShow(overlay);
                                },

                                _beforeShow: function(overlay)
                                {
                                    this.ed.$el.blur();
                                },

                                _afterShow: function(overlay)
                                {
                                    // based on XF.EditorDialogCode
                                    var $container = overlay.$container,
                                        ed = this.ed,
                                        selectedText,
                                        bbCodeValue = $container.find('[data-bb-value]:not([data-no-fill])');

                                    if (!bbCodeValue.length) {
                                        return;
                                    }

                                    if (ed.selection.isCollapsed())
                                    {
                                        selectedText = '';
                                    }
                                    else
                                    {
                                        selectedText = ed.selection.text()
                                    }

                                    // weird FF behavior where inserting code wouldn't replace the current selection without this
                                    ed.selection.save();

                                    bbCodeValue.first().val(selectedText).focus();
                                },

                                submit: function (e)
                                {
                                    e.preventDefault();

                                    var $targetForm = $(e.target).closest('form'),
                                        outputInput = $targetForm.find('[name="bb_code_output"]'),
                                        value = outputInput.val(),
                                        respectCursor = outputInput.data('bb-code-respect-cursor') === 'true';

                                    if (value)
                                    {
                                        this.insertOutput(value, outputInput, respectCursor);
                                    }

                                    this.overlay.hide();
                                },

                                insertOutput: function(value, outputInput = null, respectCursor = false) {
                                    var ed = this.ed;

                                    ed.selection.restore();
                                    ed.undo.saveStep();
                                    if (respectCursor) {
                                        ed.selection.save();
                                        var $markers = ed.$el.find('.fr-marker');
                                        $markers.first().before(outputInput.data('bb-code-start'));
                                        $markers.last().after(outputInput.data('bb-code-end'));
                                        ed.selection.restore();
                                        ed.html.insert('');
                                    } else {
                                        ed.html.insert(value);
                                    }
                                    ed.placeholder.hide();
                                    ed.undo.saveStep();
                                    SV.AdvancedBBCode.EditorButtons.normalizeAfterInsert(ed);

                                    ed.events.trigger("charCounter.update");
                                },
                            });
                        },

                        callback: function ()
                        {
                            XF.EditorHelpers.loadDialog(this, element);
                        }
                    };
                }
                SV.AdvancedBBCode[element].editorButton.init();
                XF.EditorHelpers.dialogs[element] = new SV.AdvancedBBCode[element].EditorDialog(element);

                if ($.FE.COMMANDS['xfCustom_' + element])
                {
                    $.FE.COMMANDS['xfCustom_' + element].callback = SV.AdvancedBBCode[element].editorButton.callback;
                }
            });
        },

        setupShimQuote: function(cmd) {
            cmd.callback = function () {
                var ed = this;
                ed.selection.save();
                ed.undo.saveStep();

                var html = ed.html.getSelected();
                ed.html.insert('<p>[QUOTE]</p>' + html + '<p>[/QUOTE]</p><p><br></p>');

                ed.selection.restore();
                ed.placeholder.hide();
                ed.undo.saveStep();

                SV.AdvancedBBCode.EditorButtons.normalizeAfterInsert(ed);
            };
        },

        setupShimParagraphStyle: function(cmd) {
            var oldCallback = cmd.callback;
            cmd.callback = function (c, tag) {
                var ed = this;
                oldCallback.apply(ed, [c, tag]);
                // patch css
                ed.$el.find('h1:not(.bbc-header),h2:not(.bbc-header),h3:not(.bbc-header),h4:not(.bbc-header)').addClass('bbc-header');
                ed.$el.find('p.bbc-header, span.bbc-header, div.bbc-header').removeClass('bbc-header');
            };
        },

        setupShimInsertHr: function(cmd) {
            if ($.FE.COMMANDS['xfCustom_hr']) {
                cmd.callback = $.FE.COMMANDS['xfCustom_hr'].callback
            }
        },

        normalizeAfterInsert: function(ed) {
            if ('normalizeAfterInsert' in XF.EditorHelpers){
                XF.EditorHelpers.normalizeAfterInsert(ed);
            }

            var selected = ed.html.getSelected();
            if (/<br>\s*<\/p>/.test(selected)) {
                SV.AdvancedBBCode.EditorButtons.normalizeBrForEditor(ed.$el);
                // remove the last undo step and replace it with the corrected html version
                ed.undo_index--;
                ed.undo_stack.pop();
                ed.undo.saveStep();
            }
        },

        shimInsertCode: function() {
            var originalFn = XF.EditorHelpers.insertCode;
            XF.EditorHelpers.insertCode = function(ed, type, code) {
                originalFn(ed, type, code);
                SV.AdvancedBBCode.EditorButtons.normalizeAfterInsert(ed);
            }
        },

        shimWrapSelectionText: function() {
            var originalFn = XF.EditorHelpers.wrapSelectionText;
            XF.EditorHelpers.wrapSelectionText = function(ed, before, after, save) {
                originalFn(ed, before, after, save);
                SV.AdvancedBBCode.EditorButtons.normalizeAfterInsert(ed);
            }
        },

        shimToggleBbCode: function() {
            var originalFn = $.FE.PLUGINS.bbCode;
            var xf21Compat = !('isElementWithinDraftForm' in XF);
            $.FE.PLUGINS.bbCode = function(ed) {
                var obj = originalFn(ed);
                if ('btnsToDisable' in obj) {
                    obj.btnsToDisable = function($button)
                    {
                        return ed.$tb.find('> .fr-btn-grp .fr-command, > .fr-more-toolbar .fr-command')
                            .not($button)
                            .not('[data-cmd="fullscreen"]')
                            .not('[data-cmd^="xfDraft"]')
                            .not('[data-cmd^="more"]')
                            .not('[data-cmd=xfPreview]');
                    }
                } else {
                    // copied from xf\editor.js
                    // the entire $.FE.PLUGINS.bbCode needs replacing to correctly shim btnsToDisable to skip draft buttons :(
                    var func = function(ed)
                    {
                        var _isBbCodeView = false;

                        function getButton()
                        {
                            return ed.$tb.find('.fr-command[data-cmd=xfBbCode]');
                        }

                        function getBbCodeBox()
                        {
                            var $oel = ed.$oel;

                            var $bbCodeBox = $oel.data('xfBbCodeBox');
                            if (!$bbCodeBox)
                            {
                                var borderAdjust = parseInt(ed.$wp.css('border-bottom-width'), 10)
                                    + parseInt(ed.$wp.css('border-top-width'), 10);

                                $bbCodeBox = $('<textarea class="input" style="display: none" />');
                                $bbCodeBox.attr('aria-label', XF.htmlspecialchars(XF.phrase('rich_text_box')));
                                $bbCodeBox.css({
                                    minHeight: ed.opts.heightMin ? (ed.opts.heightMin + borderAdjust) + 'px' : null,
                                    maxHeight: ed.opts.heightMax ? ed.opts.heightMax + 'px' : null,
                                    height: ed.opts.height ? (ed.opts.height + borderAdjust) + 'px' : null,
                                    padding: ed.$el.css('padding')
                                });
                                $bbCodeBox.attr('name', $oel.data('original-name'));
                                $oel.data('xfBbCodeBox', $bbCodeBox);
                                ed.$wp.after($bbCodeBox[0]);

                                $bbCodeBox.on('focus blur', function(e)
                                {
                                    switch (e.type)
                                    {
                                        case 'focus':
                                            ed.$box.addClass('is-focused');
                                            break;

                                        case 'blur':
                                            ed.$box.removeClass('is-focused');
                                            break;
                                    }
                                });

                                XF.Element.applyHandler($bbCodeBox, 'textarea-handler');
                                XF.Element.applyHandler($bbCodeBox, 'user-mentioner');
                                XF.Element.applyHandler($bbCodeBox, 'emoji-completer');

                                if (!xf21Compat && XF.isElementWithinDraftForm($bbCodeBox))
                                {
                                    XF.Element.applyHandler($bbCodeBox, 'draft-trigger');
                                }
                            }

                            return $bbCodeBox;
                        }

                        function btnsToDisable($button)
                        {
                            return ed.$tb.find(xf21Compat ? ' > .fr-command' : '> .fr-btn-grp .fr-command, > .fr-more-toolbar .fr-command')
                                .not($button)
                                .not('[data-cmd="fullscreen"]')
                                .not('[data-cmd^="xfDraft"]')
                                .not('[data-cmd^="more"]')
                                .not('[data-cmd=xfPreview]');
                        }

                        function toBbCode(bbCode, skipFocus)
                        {
                            var self = this, $bbCodeBox = self.getBbCodeBox()

                            var apply = function(bbCode, skipFocus)
                            {
                                _isBbCodeView = true;

                                var $button;

                                ed.undo.saveStep();
                                ed.$el.blur();

                                $button = self.getButton();

                                self.btnsToDisable($button).addClass('fr-disabled');
                                $button.addClass('fr-active');

                                ed.$wp.css('display', 'none');
                                ed.$oel.attr('disabled', 'disabled'); // Froala jQuery doesn't implement prop

                                $bbCodeBox.val(bbCode)
                                    .css('display', '')
                                    .prop('disabled', false)
                                    .trigger('autosize');

                                if (!skipFocus)
                                {
                                    $bbCodeBox.autofocus();
                                }

                                XF.setIsEditorEnabled(false);
                            };

                            if (typeof bbCode == 'string')
                            {
                                apply(bbCode, skipFocus);
                            }
                            else
                            {
                                XF.ajax('POST',
                                    XF.canonicalizeUrl('index.php?editor/to-bb-code'),
                                    { html: ed.html.get() },
                                    function (data) { apply(data.bbCode, skipFocus); }
                                );
                            }
                        }

                        function toHtml(html)
                        {
                            var self = this, $bbCodeBox = self.getBbCodeBox()

                            var apply = function(html)
                            {
                                _isBbCodeView = false;

                                var $button = getButton();

                                self.btnsToDisable($button).removeClass('fr-disabled');
                                $button.removeClass('fr-active');

                                ed.$oel.removeAttr('disabled'); // Froala jQuery doesn't implement prop
                                ed.html.set(html);
                                $bbCodeBox.css('display', 'none').prop('disabled', true);
                                ed.$wp.css('display', '');
                                ed.events.focus();
                                ed.undo.saveStep();
                                ed.size.refresh();

                                XF.setIsEditorEnabled(true);
                                XF.layoutChange();
                            };

                            if (typeof html == 'string')
                            {
                                apply(html);
                            }
                            else
                            {
                                var params = { bb_code: $bbCodeBox.val() };

                                var $form = ed.$el.closest('form');
                                if ($form.length)
                                {
                                    if ($form[0][ed.opts.xfBbCodeAttachmentContextInput])
                                    {
                                        params.attachment_hash_combined = $($form[0][ed.opts.xfBbCodeAttachmentContextInput]).val();
                                    }
                                }

                                XF.ajax('POST',
                                    XF.canonicalizeUrl('index.php?editor/to-html'),
                                    params,
                                    function (data) { apply(data.editorHtml); }
                                );
                            }
                        }

                        function toggle()
                        {
                            if (_isBbCodeView)
                            {
                                this.toHtml();
                            }
                            else
                            {
                                this.toBbCode();
                            }
                        }

                        function isBbCodeView()
                        {
                            return _isBbCodeView;
                        }

                        function getToggleableButtons()
                        {
                            return this.btnsToDisable(getButton());
                        }

                        function insertBbCode(bbCode)
                        {
                            if (!_isBbCodeView)
                            {
                                return;
                            }

                            var $bbCodeBox = this.getBbCodeBox();
                            XF.insertIntoTextBox($bbCodeBox, bbCode);
                        }

                        function replaceBbCode(bbCode)
                        {
                            if (!_isBbCodeView)
                            {
                                return;
                            }

                            var $bbCodeBox = this.getBbCodeBox();
                            XF.replaceIntoTextBox($bbCodeBox, bbCode);
                        }

                        function getTextArea()
                        {
                            return (_isBbCodeView ? this.getBbCodeBox() : null);
                        }

                        function _init()
                        {
                            ed.events.on('buttons.refresh', function()
                            {
                                return !_isBbCodeView;
                            });
                        }

                        return {
                            _init: _init,
                            btnsToDisable: btnsToDisable,
                            getButton: getButton,
                            getBbCodeBox: getBbCodeBox,
                            toBbCode: toBbCode,
                            isBbCodeView: isBbCodeView,
                            getTextArea: getTextArea,
                            insertBbCode: insertBbCode,
                            replaceBbCode: replaceBbCode,
                            toHtml: toHtml,
                            toggle: toggle,
                            getToggleableButtons: getToggleableButtons
                        };
                    };

                    obj = func(ed);
                }
                return obj;
            };
        },

        shimToggleFullscreen: function() {
            var originalFn = $.FE.PLUGINS.fullscreen;
            $.FE.PLUGINS.fullscreen = function(ed) {
                var obj = originalFn(ed);
                var originalToggleFunc = obj.toggle;
                obj.toggle = function() {
                    var ret = originalToggleFunc();

                    var $editor = $(ed.$box.get(0));
                    var $textarea = $editor.find('textarea.input');
                    if ($editor.length !== 0 && $textarea.length !== 0) {
                        if (!obj.isActive()) {
                            var $container = $editor.closest('.message-editorWrapper');
                            if ($container.length) {
                                XF.focusEditor($container);
                                // focus *again* to move a little once the stick-nav becomes visible
                                if ($(".p-navSticky").length !== 0) {
                                    setTimeout(function () {
                                        XF.focusEditor($container);
                                    }, 0);
                                }
                            }

                            var minHeight = $textarea.data('min-height');
                            var maxHeight = $textarea.data('max-height');
                            if (minHeight && maxHeight) {
                                $textarea.css({
                                    'min-height': minHeight,
                                    'max-height': maxHeight
                                });
                                $textarea.data('min-height', 0);
                                $textarea.data('max-height', 0);
                                // force height re-calc
                                $textarea.val($textarea.val());
                            }
                        } else {
                            // resize <textarea> to match the screen (for XF2.1)
                            $textarea.data('min-height', $textarea.css('min-height'));
                            $textarea.data('max-height', $textarea.css('max-height'));
                            $textarea.css({
                                'height': '100%',
                                'min-height': '100%',
                                'max-height': '100%'
                            });
                        }
                    }

                    return ret;
                }

                return obj;
            };
        },

        setupMonkeyPatching: function() {
            var config = SV.AdvancedBBCode.EditorButtons;

            if ($.FE.COMMANDS['xfQuote'] && SV.AdvancedBBCode.editorConfig.pasteQuotesAsBbCode) {
                config.setupShimQuote($.FE.COMMANDS['xfQuote']);
            }
            if ($.FE.COMMANDS['paragraphFormat'] && SV.AdvancedBBCode.editorConfig.shimH1) {
                config.setupShimParagraphStyle($.FE.COMMANDS['paragraphFormat']);
            }
            if ($.FE.COMMANDS['insertHR'] && SV.AdvancedBBCode.editorConfig.shimHr) {
                config.setupShimInsertHr($.FE.COMMANDS['insertHR']);
            }
            if (SV.AdvancedBBCode.editorConfig.extraBrSanitization) {
                config.shimInsertCode();
                config.shimWrapSelectionText();
            }
            if ($.FE.PLUGINS.bbCode) {
                config.shimToggleBbCode();
            }
            if ($.FE.PLUGINS.fullscreen) {
                config.shimToggleFullscreen();
            }
        },

        init: function ()
        {
            var $editorConfig = $('.js-editorConfig-ex').first();
            if ($editorConfig.length === 0) {
                console.log("Failed extend editor.");
                return;
            }

            var config = SV.AdvancedBBCode.EditorButtons;

            config.setupConfig($editorConfig);
            config.setupLanguage();
            config.setupBlockTags();
            config.setupCommands();
            config.setupMonkeyPatching();
        },
    };

    XF.Element.extend('textarea-handler', {
        __backup: {
            'init': 'svAdvancedBbCode_init'
        },

        setupDisableClickerWatcher: function(editor, disableBbCodeModeClicks, disableDisableBbCodeModeClicks) {
            var ed = editor.ed,
                enable = true;
            ed.svDisabledClicks = 0;
            ed.svDisabledClicksAttempts = 0;
            ed.$tb.on('click', 'button.fr-disabled', function() {
                if(enable && ed.bbCode.isBbCodeView()) {
                    ed.svDisabledClicks += 1;
                    if(ed.svDisabledClicks >= disableBbCodeModeClicks) {
                        ed.svDisabledClicks = 0;
                        if(confirm(XF.phrase('svAdvancedBbCodesPack_bbcode_mode_button_usage'))) {
                            ed.bbCode.toggle();
                        } else {
                            ed.svDisabledClicksAttempts += 1;
                            if (ed.svDisabledClicksAttempts >= disableDisableBbCodeModeClicks) {
                                enable = false;
                            }
                        }
                    }
                }
            });
        },

        init: function() {
            // detect if this is bb-code mode attached to a froala editor. If so, proxy update events to the word-counter
            var self = this,
                editor = null,
                $container = this.$target.closest('.message-editorWrapper');
            if ($container.length) {
                editor = XF.getEditorInContainer($container);
            }
            if (!editor) {
                $container = this.$target.closest('.bbWrapper').parent();
                if ($container.length) {
                    editor = XF.getEditorInContainer($container);
                }
            }
            if (editor && XF.Editor && editor instanceof XF.Editor) {
                editor.ed.events.on('html.set', function(){
                    editor.ed.opts.wordCounterGetText = null;
                });
                this.$target.on('autosize', function () {
                    if (!self.$target.prop('disabled')) {
                        if (!editor.ed.opts.wordCounterGetText) {
                            // patch the option to allow shimming the source text
                            editor.ed.opts.wordCounterGetText = function () {
                                return self.$target.val();
                            };
                        }
                        editor.ed.events.trigger('charCounter.update');
                    } else {
                        editor.ed.opts.wordCounterGetText = null;
                    }
                });

                // doesn't work reliably for IE9 or older, who cares.
                this.$target.on('input', function () {
                    if (editor.ed.opts.wordCounterGetText) {
                        editor.ed.events.trigger('charCounter.update');
                    }
                })

                var disableBbCodeModeClicks = SV.AdvancedBBCode.editorConfig.disableBbCodeModeClicks | 0;
                var disableDisableBbCodeModeClicks = SV.AdvancedBBCode.editorConfig.disableDisableBbCodeModeClicks | 0;
                var disableBbCodeModeClicksPhrase = SV.AdvancedBBCode.editorConfig.disableBbCodeModeClicksPhrase || '';
                if (disableBbCodeModeClicks > 0 && disableBbCodeModeClicksPhrase) {
                    XF.phrases['svAdvancedBbCodesPack_bbcode_mode_button_usage'] = disableBbCodeModeClicksPhrase;
                    this.setupDisableClickerWatcher(editor, disableBbCodeModeClicks, disableDisableBbCodeModeClicks);
                }
            }

            this.svAdvancedBbCode_init();
        },
    });

    XF.Element.extend('editor', {
        __backup: {
            'insertContent': 'svAdvancedBbCode_insertContent',
        },
        insertContent: function (html, text) {
            // noinspection Annotator
            this.svAdvancedBbCode_insertContent(html, text);
            if (SV.AdvancedBBCode.editorConfig.extraBrSanitization && !this.isBbCodeView())
            {
                SV.AdvancedBBCode.EditorButtons.normalizeAfterInsert(this.ed);
            }
        }
    });

    $(document).on('editor:config', SV.AdvancedBBCode.EditorButtons.config);
    $(document).on('editor:first-start', SV.AdvancedBBCode.EditorButtons.init);
    $(document).on('editor:init', SV.AdvancedBBCode.EditorButtons.editorInit);
})(jQuery, window, document);
