/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

var SV = window.SV || {};

(/** @param {jQuery} $ jQuery Object */ function($, window, document, _undefined)
{
  "use strict";

  /**
   * A slider for use in rich text messages.
   */
  SV.Slider = XF.Element.newHandler({
    options: {
      allowTouch: false,
      autoplay:   false,
      interval:   3000,
      cmd:        false,
      noclick:    false,
      num:        false
    },

    Swiper: null,
    $cmd:   null,

    /**
     * Initializes the carousel and binds event listeners.
     */
    init: function()
    {
      var $container = this.$target.find('.swiper-container');
      var $slides = this.$target.find('.swiper-slide');
      var $pagination = this.$target.find('.swiper-pagination');
      var $buttonPrev = this.$target.find('.swiper-button-prev');
      var $buttonNext = this.$target.find('.swiper-button-next');

      // initial slide
      var initialSlide = 0;
      var $openSlides = $slides.filter('[data-open="true"]');
      if ($openSlides.length) {
        initialSlide = $slides.index($openSlides.last());
      }

      // numerical pagination
      var renderBullet = null;
      if (this.options.num) {
        renderBullet = function (index, className) {
          return '<span class="' + className + '">' + (index + 1) + '</span>';
        };
      }

      this.Swiper = new Swiper($container, {
        initialSlide:   initialSlide,
        loop:           true,
        allowTouchMove: this.options.allowTouch,
        autoplay:       {delay: this.options.interval},

        pagination: {
          el:           $pagination,
          clickable:    true,
          renderBullet: renderBullet
        },

        navigation: {
          prevEl: $buttonPrev,
          nextEl: $buttonNext
        },

        accessibility: {
          prevSlideMessage:        XF.phrase('sv_bbcode_prev_slide'),
          nextSlideMessage:        XF.phrase('sv_bbcode_next_slide'),
          firstSlideMessage:       XF.phrase('sv_bbcode_first_slide'),
          lastSlideMessage:        XF.phrase('sv_bbcode_last_slide'),
          paginationBulletMessage: XF.phrase('sv_bbcode_pagination_bullet')
        }
      });

      // stop autoplay if not enabled
      if (this.Swiper.autoplay.running && !this.options.autoplay) {
        this.Swiper.autoplay.stop();
      }

      // setup cmd support
      if (this.options.cmd) {
        this.$cmd = this.$target.find('.swiper-cmd');
        this.updateCmd();

        this.$cmd.on('click', XF.proxy(this, 'autoplayToggle'));
        this.Swiper.on(
          'autoplayStart autoplayStop',
          XF.proxy(this, 'updateCmd')
        );
      }

      // register event handlers
      this.Swiper.slides.on('click', XF.proxy(this, 'slideClick'));

      var self = this;

      // sliders within spoilers
      var $spoiler = this.$target.closest('.bbCodeSpoiler-content');
      if ($spoiler.length) {
        $spoiler.one('toggle:shown', function() {
          self.Swiper.update();
        });
      }

      // required to correctly initialize on post insert/edit
      setTimeout(function() {
        self.Swiper.update();
      }, XF.config.speed.slow);
    },

    /**
     * @param {MouseEvent} e
     */
    slideClick: function(e)
    {
      if (this.options.noclick) {
        return;
      }

      this.Swiper.slideNext();
    },

    /**
     * @param {MouseEvent} e
     */
    autoplayToggle: function(e)
    {
      var Swiper = this.Swiper;

      if (Swiper.autoplay.running) {
        Swiper.autoplay.stop();
      } else {
        Swiper.autoplay.start();
      }
    },

    /**
     * Updates the play/pause control state.
     */
    updateCmd: function()
    {
      var $buttonPlay = this.$cmd.find('.swiper-cmd-play');
      var $buttonPause = this.$cmd.find('.swiper-cmd-pause');

      if (this.Swiper.autoplay.running) {
        $buttonPlay.hide();
        $buttonPause.show();
      } else {
        $buttonPause.hide();
        $buttonPlay.show();
      }
    }
  });

  // register handlers
  XF.Element.register('sv-slider', 'SV.Slider');
})(jQuery, window, document);
