/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

var SV = window.SV || {};

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined)
{
    "use strict";

    /**
     * A tab for use in rich text messages.
     */
    SV.Tab = XF.Click.newHandler({
        eventNameSpace: 'SVTab',

        options: {
            tabContainerId: '',
            tabId: ''
        },

        /**
         * Initializes the tab.
         */
        init: function ()
        {
        },

        /**
         * @param {MouseEvent} e
         */
        click: function (e)
        {
            var $container = $(`#${this.options.tabContainerId}`);
            var $tabs = $container.find('.sv-tabs-tab');
            var $panes = $container.find('.sv-tabs-pane');

            var $tab = this.$target;
            var $pane = $panes.filter(`#${this.options.tabId}`);

            $tabs.each(function ()
            {
                var $this = $(this);
                if ($tab.parent().parent().attr('id') === $this.parent().parent().attr('id'))
                {
                    $this.removeClass('sv-tabs-tab--active');
                }
            });

            $panes.each(function ()
            {
                var $this = $(this);
                if ($pane.parent().parent().attr('id') === $this.parent().parent().attr('id'))
                {
                    $this.removeClass('sv-tabs-pane--active');
                }
            });

            $tab.addClass('sv-tabs-tab--active');
            $pane.addClass('sv-tabs-pane--active');
        },
    });

    SV.TabSimple = XF.Element.newHandler({
        options: {
            tabContainerId: null,
            tabId: null
        },

        init: function ()
        {
            this.$target.on('click', XF.proxy(this, 'click'));
        },

        /**
         * @param {MouseEvent} e
         */
        click: function (e)
        {
            if (!this.options.tabContainerId)
            {
                console.error('No tab container provided.');
                return;
            }

            if (!this.options.tabId)
            {
                console.error('No tab provided.');
                return;
            }

            var tab = $('#' + this.options.tabContainerId + ' [data-tab-id="' + this.options.tabId + '"].sv-tabs-tab');
            if (tab.length)
            {
                tab.click();
            }
        },
    });

    // register handlers
    XF.Click.register('sv-tabs-tab', 'SV.Tab');
    XF.Element.register('sv-tabs-tab-link', 'SV.TabSimple')
})(jQuery, window, document);
