/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

var SV = window.SV || {};

(/** @param {jQuery} $ jQuery Object */ function($, window, document, _undefined)
{
    "use strict";

    SV.TransparentAsSpoiler = XF.Element.newHandler({
        options:{
            svShowIcon : null,
            svHideIcon : null,
            storageContainer: 'toggleTransparencyToSpoiler',
            storageKey: 'toggle'
        },
        colorSpans: null,
        initSpoiler: false,

        init: function() {
            var $e = this.$target;

            this.colorSpans = this.findSpans(this.findMessageBody($e));
            if (this.colorSpans.length) {
                var toggleValue = XF.ToggleStorageData.get(this.options.storageContainer, this.options.storageKey);

                var $showHide = $e.prepend($('<span />', {
                    class: (toggleValue ? this.options.svHideIcon : this.options.svShowIcon) + ' sv-transparent-as-spoiler',
                }).on('click', XF.proxy(this, 'click')));

                if (toggleValue) {
                    this.showSpoiler();
                }
            }
        },

        findMessageBody: function($e)
        {
            return $e.closest('.message-main').find('.message-userContent .message-body');
        },

        findSpans: function (messageBody){

            var backgroundColor = messageBody.css('background-color');
            return messageBody.find('.fixed-color').filter(function () {
                var color = this.style.color;
                if (color === 'transparent') {
                    return true;
                }
                if (color == backgroundColor) {
                    return true;
                }
                return false;
            });
        },

        showSpoiler: function(){
            var self = this;
            // transparent colors => spoiler. yes; white space sensitive :(
            this.colorSpans
                .addClass('bbc-spoiler')
                .addClass('bbc-spoiler-instant')
                .data('class', 'bbc-spoiler')
                .data('xfClick','toggle-class')
                .attr('tabindex', '0')
                .attr('data-xf-click','toggle-class');
        },

        hideSpoiler: function(){
            // spoiler => transparent colors
            this.colorSpans
                .removeClass('bbc-spoiler')
                .removeClass('bbc-spoiler-instant')
                .data('class', '')
                .data('xfClick','')
                .removeAttr('tabindex')
                .removeAttr('data-xf-click');
        },

        click: function(e){
            if (!this.colorSpans) {
                return;
            }
            var element = $(e.target),
                showIcon = this.options.svShowIcon,
                hideIcon = this.options.svHideIcon;
            if (element.hasClass(showIcon)) {
                element.removeClass(showIcon);
                element.addClass(hideIcon);
                // expiry for 1 month instead of 4 hours
                XF.ToggleStorageData.set(this.options.storageContainer, this.options.storageKey, 1, 2592000);
                this.showSpoiler();
            } else {
                element.removeClass(hideIcon);
                element.addClass(showIcon);
                XF.ToggleStorageData.remove(this.options.storageContainer, this.options.storageKey);
                this.hideSpoiler();
            }
        },
    });

    XF.Element.register('sv-transparent-as-spoiler', 'SV.TransparentAsSpoiler');
}) (jQuery, window, document);