<?php
/**
 * @noinspection PhpUnusedParameterInspection
 */

namespace SV\AdvancedBbCodesPack\BbCode\ProcessorAction;

use XF\BbCode\ProcessorAction\AnalyzerHooks;
use XF\BbCode\ProcessorAction\AnalyzerInterface;
use XF\BbCode\ProcessorAction\FiltererHooks;
use XF\BbCode\ProcessorAction\FiltererInterface;

/**
 * Class AnalyzeModBbCodeUsage
 *
 * @package SV\AdvancedBbCodesPack\BbCode\ProcessorAction
 */
class AnalyzeModBbCodeUsage implements AnalyzerInterface, FiltererInterface
{
    /**
     * @var string
     */
    public $title;

    /**
     * @var string
     */
    public $tagType;
    /**
     * @var int
     */
    protected $quotes = 0;

    public function addFiltererHooks(FiltererHooks $hooks)
    {
        $hooks->addSetupHook('setup')
              ->addTagHook('quote', 'filterQuoteTag');
    }

    /**
     * @param AnalyzerHooks $hooks
     */
    public function addAnalysisHooks(AnalyzerHooks $hooks)
    {
        $hooks->addTagHook('quote', 'analyzeQuoteTag');

        $tags = \explode(',', \XF::options()->modInterruptTags ?? '');
        $tags = \array_map('\trim', $tags);
        $tags = \array_filter($tags);
        $tags = \array_map('\utf8_strtolower', $tags);

        foreach ($tags as $tag)
        {
            $hooks->addTagHook($tag, 'analyzeModTag');
        }
    }

    public function setup()
    {
        $this->quotes = 0;
        $this->title = null;
        $this->tagType = null;
    }

    public function filterQuoteTag(array $tag, array $options)
    {
        $this->quotes++;
    }

    public function analyzeQuoteTag(array $tag, array $options)
    {
        $this->quotes--;
    }

    /**
     * @param array $tag
     * @param array $options
     */
    public function analyzeModTag(array $tag, array $options)
    {
        if ($this->quotes === 0 && $this->title === null)
        {
            $option = $tag['option'] ?? '';
            if (\is_array($option))
            {
                $option = $option['title'] ?? \reset($option);
            }
            $this->tagType = $tag['tag'];
            $this->title = $option;
        }
    }
}