<?php

namespace SV\AdvancedBbCodesPack\BbCode\Renderer;

use function property_exists;

trait StyleTrait
{
    /** @var ?bool */
    protected $svHasApp = null;
    /** @var ?bool */
    protected $svHasTemplater = null;

    protected function getStyle(): \XF\Style
    {
        // ensure this function works sanely when __get is implemented in a non-standard way
        if ($this->svHasApp === null)
        {
            $this->svHasApp = property_exists($this, 'app');
        }
        if ($this->svHasTemplater === null)
        {
            $this->svHasTemplater = property_exists($this, 'templater');
        }
        $app = $this->svHasApp ? $this->app : null;
        $templater = $this->svHasTemplater ? $this->templater : null;

        $app = $app ?? \XF::app();

        $templater = $templater ?? $app->templater();
        return $templater->getStyle() ?? $app->style();
    }

    /**
     * @param string          $propertyName
     * @param string|null     $dependsOn
     * @param string|int|null $defaultValue
     * @return string|int|null
     */
    protected function getStyleProperty(string $propertyName, string $dependsOn = null, $defaultValue = null)
    {
        $style = $this->getStyle();

        if ($dependsOn !== null)
        {
            if ($style->getProperty($dependsOn))
            {
                return $style->getProperty($propertyName, $defaultValue);
            }

            return $defaultValue;
        }

        return $style->getProperty($propertyName, $defaultValue);
    }
}