<?php

namespace SV\AdvancedBbCodesPack\BbCode\Renderer;

use SV\AdvancedBbCodesPack\BbCode\Tag\AbstractTag;
use SV\StandardLib\BypassAccessStatus;
use XF\BbCode\Renderer\EditorHtml;
use XF\BbCode\Renderer\Html;

/**
 * Class TagRenderer
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Renderer
 */
class TagRenderer
{
    protected $prefix = 'SV\AdvancedBbCodesPack\BbCode\Tag\\';
    /**
     * @var Html
     */
    protected $renderer;

    /**
     * @var string
     */
    protected $type;

    /**
     * @var \XF\App
     */
    protected $app;

    /**
     * @param Html   $renderer
     * @param string $type
     */
    public function __construct(Html $renderer, string $type)
    {
        $this->renderer = $renderer;
        $this->type = $type;
        $this->app = \XF::app();
    }

    public function bindToRenderer()
    {
        // rebind all bb-code owned by this add-on which have a static callback
        // into an object callback

        $bypassAccessStatus = new BypassAccessStatus();
        $getTags = $bypassAccessStatus->getPrivate($this->renderer, 'tags');
        /** @var array $tags */
        $tags = $getTags();

        if ($this->renderer instanceof EditorHtml)
        {
            $customBbCode = \XF::app()->bbCode()->get('custom');

            if (isset($customBbCode['sub']))
            {
                $this->renderer->addTag('sub', ['replace' => ['<sub>', '</sub>']]);
            }
            if (isset($customBbCode['sup']))
            {
                $this->renderer->addTag('sup', ['replace' => ['<sup>', '</sup>']]);
            }

            foreach ($customBbCode as $tagName => $definition)
            {
                if (isset($tags[$tagName]) &&
                    !empty($definition['callback_class']) &&
                    !empty($definition['callback_method']))
                {
                    if ($this->bindTagtoRenderer($tagName, [
                        'callback' => [$definition['callback_class'], $definition['callback_method']]
                    ], 'renderEditor'))
                    {
                        $this->renderer->modifyTag($tagName,[
                            'trimAfter' => $definition['trim_lines_after'] ?? 0,
                        ]);
                    }
                }
            }

            return;
        }

        // the [tab] & [tab_header] bbcode are special, as we don't make it a full bb-code
        if (isset($tags['tabs']))
        {
            $this->renderer->addTag('tab_header', [
                'callback' => ['SV\AdvancedBbCodesPack\BbCode\Tag\TabHeader', 'renderStatic'],
            ]);
            $this->renderer->addTag('tab', [
                'callback' => ['SV\AdvancedBbCodesPack\BbCode\Tag\Tab', 'renderStatic']
            ]);
        }
        if (isset($tags['tabs']) || isset($tags['slider']) || isset($tags['accordion']))
        {
            $this->renderer->addTag('slide_header', [
                'callback' => ['SV\AdvancedBbCodesPack\BbCode\Tag\TabHeader', 'renderStatic'],
            ]);
            $this->renderer->addTag('slide', [
                'callback' => ['SV\AdvancedBbCodesPack\BbCode\Tag\Tab', 'renderStatic']
            ]);
        }
        /** @var array $tags */
        $tags = $getTags();


        foreach ($tags as $tagName => $tag)
        {
            $this->bindTagtoRenderer($tagName, $tag);
        }
    }

    /**
     * @param string $tagName
     * @param array  $tag
     * @param string $overStampMethod
     * @return bool
     * @throws \Exception
     */
    protected function bindTagtoRenderer(string $tagName, array $tag, string $overStampMethod = ''): bool
    {
        if (empty($tag['callback']) || $tag['callback'] instanceof \Closure)
        {
            return false;
        }

        if (
            isset($tag['callback'][0])
            && \is_string($tag['callback'][0])
            && \strncmp($tag['callback'][0], $this->prefix, \strlen($this->prefix)) === 0
            && ($method = \preg_replace('/Static$/', '', $tag['callback'][1], 1))
        )
        {
            $className = $this->app->extendClass($tag['callback'][0]);
            /** @var AbstractTag $obj */
            $obj = new $className($this->app, $this->renderer, $this->type);
            if ($overStampMethod)
            {
                if (!\is_callable([$obj, $overStampMethod]))
                {
                    return false;
                }
                $method = $overStampMethod;
            }

            $this->renderer->modifyTag($tagName, ['callback' => [$obj, $method]]);

            return true;
        }

        return false;
    }
}