<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * Class Article
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Article extends AbstractTag
{
    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $tagOptions = $this->getTagOptions($tagOption, $options, false);
        $tagOptions = $this->parseTagOptions($tagOptions, $options);

        $this->trimChildrenList($tagChildren);
        $content = $this->renderEscapedSubTree($tagChildren, $options);
        $template = $this->rendererType === 'emailHtml' ? 'email:sv_bbcode_article' : 'public:sv_bbcode_article';

        return $this->templater->renderTemplate($template, [
            'content' => $content,
            'options' => $tagOptions,
        ]);
    }

    protected function parseUniqueTagOptions(array &$lowerTagOptions, array &$tagOptions, array &$options, array $renderingOptions, array $constraints)
    {
        // unique options
        foreach ($lowerTagOptions as $key => $tagOption)
        {
            if (!\is_integer($key))
            {
                switch ($key)
                {
                    case 'url':
                        $cleaned = \trim($tagOption);
                        $url = $this->getUrl($cleaned, $options);
                        if ($url)
                        {
                            $options['url'] = $url;
                            $options['source'] = $tagOption;
                        }
                        break;
                }
                continue;
            }

            $cleaned = \trim($tagOptions[$key]);
            $url = $this->getUrl($cleaned, $options);
            if ($url)
            {
                $options['url'] = $url;
                $options['source'] = $cleaned;
                continue;
            }

            // source
            if ($cleaned !== '')
            {
                $options['source'] = $cleaned;
            }
        }
    }

    public function getDefaultOptions(): array
    {
        return [
            'block_align' => 'left',
            'source'      => '',
            'url'         => ''
        ];
    }

    protected function getOptionConstraints(): array
    {
        // todo - requires css/html layout support for this
        return \array_merge(parent::getOptionConstraints(), [
            'allowBlock'  => false,
            'allowFloat'  => false,
            'allowWidth'  => false,
            'allowHeight' => false,
        ]);
    }
}