<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * Class Encadre
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Encadre extends AbstractTag
{
    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $this->trimChildrenList($tagChildren);
        $content = $this->renderEscapedSubTree($tagChildren, $options);

        $tagOptions = $this->getTagOptions($tagOption, $options);
        $tagOptions = $this->parseTagOptions($tagOptions, $options);
        $template = $this->rendererType === 'emailHtml' ? 'email:sv_bbcode_quote_chunk' : 'public:sv_bbcode_encadre';

        return $this->templater->renderTemplate($template, [
            'content' => $content,
            'options' => $tagOptions
        ]);
    }

    protected function parseUniqueTagOptions(array &$lowerTagOptions, array &$tagOptions, array &$options, array $renderingOptions, array $constraints)
    {
        foreach ($lowerTagOptions as $key => $tagOption)
        {
            if (!\is_integer($key))
            {
                switch ($key)
                {
                    case 'title':
                        $options['title'] = $tagOption;
                        break;
                    case 'align':
                    case 'title_align':
                        $this->parseTitleAlignmentOptions(\strtolower($tagOption), $options);
                        break;
                    case 'skin':
                        $skin = \strtolower($tagOption);
                        switch ($skin)
                        {
                            case '1':
                            case '2':
                                $options['skin'] = 'skin' . $skin;
                                break;
                            case 'skin1':
                            case 'skin2':
                                $options['skin'] = $skin;
                                break;
                        }

                        break;
                }
                continue;
            }

            switch ($tagOption)
            {
                case 'left':
                case 'center':
                case 'right':
                    $options['title_align'] = $tagOption;
                    continue 2;

                case 'skin2':
                    $options['skin'] = 'skin2';
                    continue 2;
            }

            $cleaned = \trim($tagOptions[$key]);
            // title
            if ($cleaned !== '')
            {
                $options['title'] = $cleaned;
            }
        }
    }

    public function getDefaultOptions(): array
    {
        $style = $this->getStyle();

        return [
            // global options
            'block_align' => $style->getProperty('svBbcodeEncadreDefaultBlockAlignment'),
            'width'       => $style->getProperty('svBbcodeEncadreDefaultWidth'),
            'width_type'  => $style->getProperty('svBbcodeEncadreDefaultWidthUnit'),

            // encadre options
            'title'       => '',
            'title_align' => $style->getProperty('svBbcodeEncadreDefaultTitleAlignment'),
            'skin'        => $style->getProperty('svBbcodeEncadreDefaultSkin')
        ];
    }

    protected function getOptionConstraints(): array
    {
        return \array_merge(parent::getOptionConstraints(), [
            'defaultWidthType'  => '%',
            'allowHeight'       => false,
            'max_width_px'      => $this->getStyleProperty('svBbcodeEncadreMaxWidthPx', 'svBbcodeEncadreApplyWidthRestrictions'),
            'max_width_percent' => $this->getStyleProperty('svBbcodeEncadreMaxWidthPercent', 'svBbcodeEncadreApplyWidthRestrictions')
        ]);
    }
}