<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * Class Fieldset
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Fieldset extends AbstractTag
{
    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $this->trimChildrenList($tagChildren);
        $content = $this->renderEscapedSubTree($tagChildren, $options);

        $tagOptions = $this->getTagOptions($tagOption, $options);
        $tagOptions = $this->parseTagOptions($tagOptions, $options);
        $template = $this->rendererType === 'emailHtml' ? 'email:sv_bbcode_quote_chunk' : 'public:sv_bbcode_fieldset';

        return $this->templater->renderTemplate($template, [
            'content' => $content,
            'options' => $tagOptions
        ]);
    }

    protected function parseUniqueTagOptions(array &$lowerTagOptions, array &$tagOptions, array &$options, array $renderingOptions, array $constraints)
    {
        foreach ($lowerTagOptions as $key => $option)
        {
            if (!\is_integer($key))
            {
                switch($key)
                {
                    case 'title':
                        $options['title'] = $option;
                        break;
                }
                continue;
            }
            $cleaned = \trim($tagOptions[$key]);

            // title
            if ($cleaned !== '')
            {
                $options['title'] = $cleaned;
            }
        }
    }

    public function getDefaultOptions(): array
    {
        $style = $this->getStyle();

        return [
            // global options
            'block_align' => $style->getProperty('svBbcodeFieldsetDefaultBlockAlignment'),
            'width'       => $style->getProperty('svBbcodeFieldsetDefaultWidth'),
            'width_type'  => $style->getProperty('svBbcodeFieldsetDefaultWidthUnit'),

            // fieldset options
            'title'       => ''
        ];
    }

    protected function getOptionConstraints(): array
    {
        return \array_merge(parent::getOptionConstraints(), [
            'defaultWidthType'  => '%',
            'allowHeight'       => false,
            'max_width_px'      => $this->getStyleProperty('svBbcodeFieldsetMaxWidthPx', 'svBbcodeFieldsetApplyWidthRestrictions'),
            'max_width_percent' => $this->getStyleProperty('svBbcodeFieldsetMaxWidthPercent', 'svBbcodeFieldsetApplyWidthRestrictions')
        ]);
    }
}