<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * Class Gview
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Gview extends AbstractTag
{
    use TagWithSingleOptionTrait;

    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $tagOptions = $this->getTagOptions($tagOption, $options, false);
        $tagOptions = $this->parseTagOptions($tagOptions, $options);

        $content = $this->renderer->renderSubTreePlain($tagChildren);
        $content = \trim($content);

        $wasWrapped = false;
        // unwrap embed code
        if (\preg_match('#^<iframe src="(https://\w+\.google\.com/[^"]+)".*?></iframe>#ui', $content, $match))
        {
            $wasWrapped = true;
            $content = $match[1];
        }

        if (\preg_match('#^(https://docs\.google\.com/document/\w{1,9}/[^/]+)(?:/pub|/edit)?(?:[?\#].*)?$#ui', $content, $match))
        {
            $finalUrl = $match[1] . '/pub?embedded=true';
        }
        else if (\preg_match('#^(https://docs\.google\.com/file/\w{1,9}/[^/]+)(?:/preview|/view)?(?:[?\#].*)?$#ui', $content, $match))
        {
            $finalUrl = $match[1] . '/preview';
        }
        else if (\preg_match('#^(https://drive\.google\.com/file/\w{1,9}/[^/]+)(?:/preview|/view)?(?:[?\#].*)?$#ui', $content, $match))
        {
            $finalUrl = $match[1] . '/preview';
        }
        else if (\preg_match('#^(https://docs\.google\.com/spreadsheets/\w{1,9}/[^/]+)(?:/pubhtml)?(?:[?\#]*)?$#ui', $content, $match))
        {
            $finalUrl = $match[1] . '/pubhtml';
        }
        else if (\preg_match('#^(https://docs\.google\.com/presentation/\w{1,9}/[^/]+)(?:/embed)?(?:[?\#].*)?#ui', $content, $match))
        {
            $finalUrl = $match[1] . '/embed';
        }
        else if (\preg_match('#^https://docs\.google\.com/viewer\?(authuser=.+)$#ui', $content, $url))
        {
            $finalUrl = 'https://docs.google.com/viewer?' . $url[1];
        }
        else
        {
            $finalUrl = 'https://docs.google.com/viewer?' . 'url=' . \rawurlencode($content) . '&embedded=true';
        }

        if (!$finalUrl)
        {
            return $wasWrapped ? "[gview]{$content}[/gview]" : $this->renderer->renderUnparsedTag($tag, $options);
        }

        if ($this->rendererType === 'emailHtml')
        {
            return $this->formatter->censorText($finalUrl);
        }

        return $this->templater->renderTemplate('public:sv_bbcode_gview', [
            'url' => $finalUrl,
            'options' => $tagOptions,
        ]);
    }

    public function getDefaultOptions(): array
    {
        $style = $this->getStyle();

        return [
            'block_align'    => 'left',
            'width'          => '100',
            'width_type'     => '%',
            'height'         => $style->getProperty('svBbcodeGviewHeight'),
            'responsiveMode' => true,
        ];
    }

    protected function getOptionConstraints(): array
    {
        return \array_merge(parent::getOptionConstraints(), [
            'defaultWidthType'  => '%',
            // todo - requires css/html layout support for this
            'allowBlock'  => false,
            'allowFloat'  => false,
        ]);
    }
}
