<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * A BB code renderer for [HR] tags.
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Hr extends AbstractTag
{
    use TagWithSingleOptionTrait;

    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $option = $this->getTagOptionAsString($tagOption);
        $content = \trim($this->renderer->renderSubTree($tagChildren, $options));

        if ($option === '' || !\is_numeric($option))
        {
            $style = $this->getStyle();
            $hrType = (int)$style->getProperty('svBbcodeDefaultHrStyle');
        }
        else
        {
            $hrType = (int)$option;
        }

        return $this->templater->renderTemplate('public:sv_bbcode_hr', [
            'hrType' => $hrType,
            'content' => \strlen($content) ? $this->templater->preEscaped("\n" . $content) : null,
        ]);
    }

    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function renderEditor(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $content = \trim($this->renderer->renderSubTree($tagChildren, $options));

        return '<break-start />' . $this->render([], $tagOption, $tag, $options) . "<break-start />\n{$content}";
    }
}