<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

/**
 * Class Latex
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Latex extends AbstractTag
{
    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $content = $this->renderer->renderSubTreePlain($tagChildren);
        $content = $this->formatter->censorText($content);
        if ($this->rendererType === 'emailHtml')
        {
            return $content;
        }
        $content = \str_replace(['\[', '\]'], '', $content);

        $tagOptions = $this->getTagOptions($tagOption, $options);
        $tagOptions = $this->parseTagOptions($tagOptions, $options);

        return $this->templater->renderTemplate('public:sv_bbcode_latex', [
            'content'        => $content,
            'options'        => $tagOptions,
        ]);
    }

    protected function parseUniqueTagOptions(array &$lowerTagOptions, array &$tagOptions, array &$options, array $renderingOptions, array $constraints)
    {
        foreach ($lowerTagOptions as $key => $option)
        {
            if (!\is_integer($key))
            {
                switch($key)
                {
                    case 'title':
                        $options['title'] = $option;
                        break;
                }
                continue;
            }
            $cleaned = \trim($tagOptions[$key]);

            // title
            if ($cleaned !== '')
            {
                $options['title'] = $cleaned;
            }
        }
    }

    public function getDefaultOptions(): array
    {
        $style = $this->getStyle();

        return [
            // global options
            'block_align' => $style->getProperty('svBbcodeLatexDefaultBlockAlignment'),
            'width'       => $style->getProperty('svBbcodeLatexDefaultWidth'),
            'width_type'  => $style->getProperty('svBbcodeLatexDefaultWidthUnit'),
            'height'      => null,

            // latex options
            'title'       => ''
        ];
    }

    protected function getOptionConstraints(): array
    {
        return \array_merge(parent::getOptionConstraints(), [
            'max_width_px'      => $this->getStyleProperty('svBbcodeLatexMaxWidthPx', 'svBbcodeLatexApplyWidthRestrictions'),
            'max_width_percent' => $this->getStyleProperty('svBbcodeLatexMaxWidthPercent', 'svBbcodeLatexApplyWidthRestrictions'),
            'max_height'        => $this->getStyleProperty('svBbcodeLatexMaxHeightPx', 'svBbcodeLatexApplyHeightRestrictions')
        ]);
    }
}