<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

use SV\AdvancedBbCodesPack\BbCode\ChildTagParser;
use XF\BbCode\RuleSet;

/**
 * Class Table
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class Table extends AbstractTag
{
    /** @var RuleSet */
    protected $tableRules;
    /** @var \SV\AdvancedBbCodesPack\BbCode\Renderer\TableHtml */
    protected $tableRenderer;
    /** @var ChildTagParser */
    protected $childTagParser;

    protected function getTableRules(): RuleSet
    {
        if ($this->tableRules)
        {
            return $this->tableRules;
        }

        $parentRules = $this->renderer->getRules();

        $class = $this->app->extendClass('XF\BbCode\RuleSet');
        /** @var \XF\BbCode\RuleSet $ruleSet */
        $ruleSet = new $class($parentRules->getContext(), 'table', false);
        $xTableTag = $parentRules->getTag('xtable');
        $tableRules = $xTableTag['childTags'] ?? [];
        foreach ($tableRules as $tagName => $config)
        {
            $ruleSet->addTag($tagName, $config);
        }

        $this->tableRules = $ruleSet;

        return $this->tableRules;
    }

    protected function getTableRenderer(): \SV\AdvancedBbCodesPack\BbCode\Renderer\TableHtml
    {
        if ($this->tableRenderer)
        {
            return $this->tableRenderer;
        }

        $this->tableRenderer = $this->app->bbCode()->renderer('SV\AdvancedBbCodesPack:TableHtml');
        $this->tableRenderer->setParentRender($this->renderer);
        $this->tableRenderer->setTemplater($this->templater);

        return $this->tableRenderer;
    }

    protected function getChildTagParser(): ChildTagParser
    {
        if ($this->childTagParser)
        {
            return $this->childTagParser;
        }

        $class = $this->app->extendClass('SV\AdvancedBbCodesPack\BbCode\ChildTagParser');

        $this->childTagParser = new $class();

        return $this->childTagParser;
    }

    /**
     * @param array           $tagChildren
     * @param null|string|string[] $tagOption
     * @param array           $tag
     * @param array           $options
     * @return string|\XF\PreEscaped
     * @throws \Exception
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $tagName = $tag['tag'];
        $tableRules = $this->getTableRules();
        $tableRenderer = $this->getTableRenderer();
        $childTagParser = $this->getChildTagParser();

        // compile to plane text, but replacing [table] with {table} so the child parser can handle nested tables properly
        $content = $this->rewriteSubTreeForChildParser($tagName, $tagChildren);
        $content = '{' . $tagName . '}' . $content . '{/' . $tagName . '}';

        $ast = $childTagParser->parse($content, $tableRules);
        if (empty($ast[0]) || !\is_array($ast[0]))
        {
            return $this->renderer->renderUnparsedTag($tag, $options);
        }

        $ast[0]['option'] = $tagOption;
        return $tableRenderer->renderTable($ast[0], $options);
    }

    /**
     * @param string $tagName
     * @param array  $tree
     * @return string|\XF\PreEscaped
     * @noinspection PhpMissingParamTypeInspection
     */
    public function rewriteSubTreeForChildParser($tagName, array $tree)
    {
        $output = '';
        foreach ($tree AS $element)
        {
            if (\is_array($element))
            {
                $prepend = $element['original'][0];
                $append = $element['original'][1];

                if ($element['tag'] === $tagName)
                {
                    $prepend = '{' . \utf8_substr($prepend, 1, \utf8_strlen($prepend) - 2) . '}';
                    $append = '{' . \utf8_substr($append, 1, \utf8_strlen($append) - 2) . '}';
                }

                $output .= $prepend
                           . $this->rewriteSubTreeForChildParser($tagName, $element['children'])
                           . $append;
            }
            else
            {
                $output .= $element;
            }
        }

        return $output;
    }
}
