<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\BbCode\Renderer;

use SV\AdvancedBbCodesPack\BbCode\Renderer\StyleTrait;
use function preg_replace, is_string;

/**
 * Extends \XF\BbCode\Renderer\EditorHtml
 */
class EditorHtml extends XFCP_EditorHtml
{
    use StyleTrait;

    public function addDefaultTags()
    {
        parent::addDefaultTags();

        // XF2.2.0 to XF2.2.5 support
        if (\XF::$versionId < 2020670 && isset($this->tags['color']['replace']))
        {
            $this->modifyTag('color', [
                'replace'  => null,
                'callback' => 'renderTagColor',
            ]);
        }
    }

    /**
     * @param array        $children
     * @param string|array $option
     * @param array        $tag
     * @param array        $options
     * @return string
     */
    public function renderTagColor(array $children, $option, array $tag, array $options)
    {
        $tagOption = \trim(\strtolower(\is_array($option) ? \implode(' ', $option) : (string)$option));
        if ($tagOption === 'transparent')
        {
            $text = $this->renderSubTree($children, $options);

            return "[COLOR=transparent]{$text}[/COLOR]";
        }

        if (\XF::$versionId < 2020670)
        {
            // XF2.2.0 to XF2.2.5 support
            $text = $this->renderSubTree($children, $options);
            if (\trim($text) === '')
            {
                return $text;
            }
            $color = \htmlspecialchars($tagOption);

            return $this->wrapHtml('<span style="color:' . $color . '">', $text, '</span>');
        }

        // XF2.2.6+ support
        return parent::renderTagColor($children, $option, $tag, $options);
    }

    public function renderTagQuote(array $children, $option, array $tag, array $options)
    {
        if ($this->getStyle()->getProperty('svDisableQuotesInEditor', false))
        {
            $renderRule = $this->tags[$tag['tag']] ?? null;
            $padding = \str_repeat("<break-start />\n", $renderRule['trimAfter'] ?? 0);

            return $this->renderUnparsedTag($tag, $options) . $padding;
        }

        return parent::renderTagQuote($children, $option, $tag, $options);
    }
}