<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\BbCode\Renderer;

use XF\Str\Formatter;
use XF\Template\Templater;
use function preg_match,preg_replace, is_string,preg_replace_callback;

/**
 * Extends \XF\BbCode\Renderer\EditorHtml
 */
class EditorHtmlPatch extends XFCP_EditorHtmlPatch
{
    public function __construct(Formatter $formatter, Templater $templater)
    {
        parent::__construct($formatter, $templater);

        if ((\XF::$versionId < 2021100 && \XF::$versionId > 2020000) || (\XF::$versionId < 2011300))
        {
            $this->blockTagsOpenRegex = '(?:<p|<div|<blockquote|<ul|<ol|<table|<h\\d|<hr)(?=[\s>]|$)';
        }
    }

    /**
     * @param string $open
     * @param string $inner
     * @param string $close
     * @param array<string>|string|null $option
     * @return string
     */
    public function wrapHtml($open, $inner, $close, $option = null)
    {
        // Note; $option isn't ever used, and contains a potential security issue
        // https://xenforo.com/community/threads/wrapping-indent-around-quote-does-not-behave-sanely-when-toggling-bb-code-mode.200346/
        // Malformed html where <p><p>...</p></p> is generated in the editor
        $newOpen = preg_replace('/^<p /i', '<div ', $open);
        if (is_string($newOpen) && $newOpen !== $open)
        {
            $open = $newOpen;
            $close = '</div>';
        }

        if (\XF::$versionId >= 2021100 || (\XF::$versionId >= 2011300 && \XF::$versionId < 2020000))
        {
            return parent::wrapHtml($open, $inner, $close);
        }

        $btOpen = $this->blockTagsOpenRegex;
        $btClose = $this->blockTagsCloseRegex;
        $callback = function (array $m) use ($close, $open) { return $close . $m[0] . $open; };

        // $open can be full of untrusted user input. Repeatedly calling preg_replace can result in an injection vulnerability in the editor
        $inner = preg_replace('#(<break />\s*)(?=<break />|$)#i', '\\1<empty-content />', $inner);

        if (preg_match('#^(' . $btOpen . ')#i', $open))
        {
            $inner = preg_replace_callback(
                '#<break-start />(?>\s*)(?!' . $btOpen . '|' . $btClose . '|$)#i',
                $callback,
                $inner
            );
            $inner = preg_replace_callback(
                '#<break />(?>\s*)(?!' . $btOpen . ')#i',
                $callback,
                $inner
            );
        }
        else
        {
            if (preg_match('#^<break />#i', $inner))
            {
                $inner = '<empty-content />' . $inner;
            }
            $inner = preg_replace_callback('#<break />\s*((' . $btOpen . ')[^>]*>)?#i', $callback, $inner);
        }

        return $open . $inner . $close;
    }

    protected function renderFinalListOutput($listType, array $elements)
    {
        $output = parent::renderFinalListOutput($listType, $elements);

        // Workaround for a bug in the editor; https://xenforo.com/community/threads/trailing-whitespace-after-link-inside-list-with-a-child-list-evaporates-in-editor.196772/
        if (\XF::$versionId < 2020700)
        {
            $output = \preg_replace_callback('/<\/a>(\s+)/i', function ($match) {
                return '</a>' . \str_repeat('&nbsp;', \utf8_strlen($match[1] ?? ''));
            }, $output);
        }

        return $output;
    }
}