<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\BbCode\Renderer;

use SV\AdvancedBbCodesPack\BbCode\Renderer\StyleTrait;

/**
 * Extends \XF\BbCode\Renderer\EmailHtml
 */
class EmailHtml extends XFCP_EmailHtml
{
    use StyleTrait;

    /** @var bool */
    protected $svEnableTransparentColor = true;

    public function addDefaultTags()
    {
        parent::addDefaultTags();

        // XF2.2.0 to XF2.2.5 support
        if (isset($this->tags['color']['replace']))
        {
            $this->modifyTag('color', [
                'replace'  => null,
                'callback' => 'renderTagColor',
            ]);
        }

        $this->svEnableTransparentColor = \XF::options()->svEnableTransparentColor ?? true;
    }

    /**
     * @param array        $children
     * @param string|array $option
     * @param array        $tag
     * @param array        $options
     * @return string
     */
    public function renderTagColor(array $children, $option, array $tag, array $options)
    {
        // SpamAssassin looks for color=transparent, rewrite to render as inline spoilers
        // https://xenforo.com/community/threads/spam-filters-regard-transparent-text-in-emails-as-spammy.190015/
        $tagOption = \trim(\strtolower(\is_array($option) ? \implode(' ', $option) : (string)$option));
        if ($tagOption === 'transparent')
        {
            if ($this->svEnableTransparentColor)
            {
                return $this->renderTagInlineSpoiler($children, $option, $tag, $options);
            }
            else
            {
                return $this->renderSubTree($children, $options);
            }
        }

        if (\XF::$versionId < 2020670)
        {
            // XF2.2.0 to XF2.2.5 support
            $text = $this->renderSubTree($children, $options);
            if (\trim($text) === '')
            {
                return $text;
            }
            $color = \htmlspecialchars($tagOption);

            return $this->wrapHtml('<span style="color:' . $color . '">', $text, '</span>');
        }

        // XF2.2.6+ support
        return parent::renderTagColor($children, $option, $tag, $options);
    }

    /**
     * @param string $content
     * @return string
     */
    protected function getRenderedInlineSpoiler($content)
    {
        if ($this->getStyleProperty('svUseNonBluredSpoiler'))
        {
            return \utf8_trim($this->templater->renderTemplate('public:sv_bbcode_spoiler', [
                'content' => $content
            ]));
        }

        return parent::getRenderedInlineSpoiler($content);
    }
}