<?php

namespace SV\AdvancedBbCodesPack\XF\Pub\Controller;

use XF\Mvc\Reply\AbstractReply;

/**
 * Extends \XF\Pub\Controller\Account
 */
class Account extends XFCP_Account
{
    public function actionEditorPlayground(): AbstractReply
    {
        if (!(\XF::options()->svEditorPlayground ?? false))
        {
            return $this->notFound();
        }

        /** @var \XF\Entity\HelpPage $helpPage */
        $helpPage = $this->finder('XF:HelpPage')
                         ->where('page_id', 'bb_codes')
                         ->where('active', 1)
                         ->whereAddOnActive()
                         ->fetchOne();

        $viewParams = [
            'helpPage' => $helpPage,
        ];

        $view = $this->view('SV\AdvancedBbCodesPack:Account\Editor', 'svAdvancedBBCode_editor_playground', $viewParams);

        return $this->addAccountWrapperParams($view, 'sv-editor-playground');
    }

    public function actionEditorPlaygroundPreview(): AbstractReply
    {
        if (!(\XF::options()->svEditorPlayground ?? false))
        {
            return $this->notFound();
        }

        $this->assertPostOnly();

        if ($this->request->exists('message'))
        {
            $editorHtml = '';
            $editorHtml2BbCode = $this->request->filter('message', 'str');
        }
        else
        {
            /** @var \XF\ControllerPlugin\Editor $editor */
            $editor = $this->plugin('XF:Editor');

            $editorHtml = $this->request->filter('message_html', 'str,no-clean');
            $editorHtml2BbCode = $editor->convertToBbCode($editorHtml);
        }

        $viewParams = [
            'showRawHtmlInput' => $this->filter('showRawHtmlInput', 'bool'),
            'editorHtml'       => $editorHtml,

            'showRawBbCodeInput' => $this->filter('showRawBbCodeInput', 'bool'),
            'editorHtml2BbCode'  => $editorHtml2BbCode,

            'showRawHtmlOutput' => $this->filter('showRawHtmlOutput', 'bool'),
        ];

        return $this->view('SV\AdvancedBbCodesPack\XF:Account\EditorPreview', 'svAdvancedBBCode_editor_playground_preview', $viewParams);
    }
}