<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\Pub\Controller;

use SV\AdvancedBbCodesPack\XF\Repository\BbCode;
use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

/**
 * Class Editor
 *
 * @package SV\AdvancedBbCodesPack\XF\Pub\Controller
 */
class Editor extends XFCP_Editor
{
    /**
     * @param $dialog
     * @return array
     */
    protected function loadDialog($dialog)
    {
        $data = parent::loadDialog($dialog);

        /** @var BbCode $bbCodeRepo */
        $bbCodeRepo = \XF::repository('XF:BbCode');
        $editorConfig = $bbCodeRepo->getSvEditorConfig();

        if (\in_array($data['dialog'], $editorConfig['dialog'], true))
        {
            if (\in_array($data['dialog'], $editorConfig['modBbCode'], true))
            {
                $data['template'] = 'svAdvancedBBCode_editor_dialog_mod';
                $data['params']['mod'] = $data['dialog'];
            }
            else
            {
                $data['template'] = 'svAdvancedBBCode_editor_dialog_' . $data['dialog'];
            }
            $data['params'] = \array_merge($data['params'], $this->getSvBbCodeContainerData());
        }

        return $data;
    }

    /**
     * @return array
     */
    protected function getSvBbCodeContainerData()
    {
        $requestUri = $this->filter('_xfRequestUri', 'str');
        $routePath = $this->request()->getRoutePathFromUrl($requestUri);
        $routeMatch = $this->router()->routeToController($routePath);
        $parameterBag = $routeMatch->getParameterBag();
        $controller = $routeMatch->getController();
        $containerData = [];

        $data = $this->getSvBbCodePreviewInfo($controller, $parameterBag, $requestUri);
        if ($data)
        {
            if (empty($data['canView']))
            {
                $data['canView'] = 'canView';
            }

            /** @var Entity $container */
            $container = $this->em()->find($data['shortName'], $data['identifier'], $data['extraWith']);
            if (!$container)
            {
                return $containerData;
            }
            if (!\method_exists($container, $data['canView']))
            {
                throw new \LogicException("getSvBbCodePreviewInfo requires to set {$data['canView']} for {$controller}");
            }

            $error = null;
            if (!$container->{$data['canView']}($error))
            {
                // just bail without setting the preview url
                return $containerData;
            }

            $svBbCodePreviewUrl = null;
            $previewRoute = $data['previewRoute'] ?? '';
            if (\strlen($previewRoute))
            {
                $svBbCodePreviewUrl = $this->buildLink($previewRoute, $container);
            }

            $containerData['svBbCodePreviewUrl'] = $svBbCodePreviewUrl;
            $containerData['svBbCodeContainer'] = $container;
        }

        return $containerData;
    }

    /**
     * @param string       $controller
     * @param ParameterBag $parameterBag
     * @param string       $requestUri
     * @return array
     * @noinspection PhpUnusedParameterInspection
     */
    protected function getSvBbCodePreviewInfo(string $controller, ParameterBag $parameterBag, string $requestUri)
    {
        $visitor = \XF::visitor();
        switch ($controller)
        {
            case 'XF:Thread':
                return [
                    'shortName'    => 'XF:Thread',
                    'identifier'   => $parameterBag->get('thread_id'),
                    'previewRoute' => 'threads/sv-bbcode-preview',
                    'extraWith'    => [
                        'Forum', 'Forum.Node', 'Forum.Node.Permissions|' . $visitor->permission_combination_id
                    ]
                ];
            case 'XF:Forum':
                return [
                    'shortName'    => 'XF:Forum',
                    'identifier'   => $parameterBag->get('node_id'),
                    'previewRoute' => 'forums/sv-bbcode-preview',
                    'extraWith'    => [
                        'Node', 'Node.Permissions|' . $visitor->permission_combination_id
                    ]
                ];
            case 'XF:Conversation':
                if (!$visitor->user_id)
                {
                    return null;
                }
                $isExisting = $parameterBag->offsetExists('conversation_id');
                if (!$isExisting)
                {
                    return [
                        'shortName'    => 'XF:User',
                        'identifier'   => $visitor->user_id,
                        'canView'      => 'canStartConversation',
                        'previewRoute' => 'conversations/sv-bbcode-new-preview',
                        'extraWith'    => [],
                    ];
                }

                return [
                    'shortName'    => 'XF:ConversationMaster',
                    'identifier'   => $parameterBag->get('conversation_id'),
                    'previewRoute' => 'conversations/sv-bbcode-preview',
                    'extraWith'    => [
                        'Users|' . $visitor->user_id
                    ]
                ];
            default:
                return null;
        }
    }
}