<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\Service\Message;

/**
 * Extends \XF\Service\Message\Preparer
 */
class Preparer extends XFCP_Preparer
{
    /**
     * @var bool
     */
    protected $checkForModBbCodeUsage = false;

    /**
     * @param string $message
     * @param bool   $checkValidity
     * @return string
     */
    public function prepare($message, $checkValidity = true)
    {
        $threadmarkCategoryId = (int)($this->app->options()->modInterruptThreadmarkCategory ?? 0);
        $this->checkForModBbCodeUsage = $this->canCheckForModBbCodeUsage(
            $threadmarkCategoryId
        );

        try
        {
            $message = parent::prepare($message, $checkValidity);

            if ($this->checkForModBbCodeUsage)
            {
                /** @var \SV\AdvancedBbCodesPack\BbCode\ProcessorAction\AnalyzeModBbCodeUsage $modBbCodeUsage */
                $modBbCodeUsage = $this->bbCodeProcessor->getAnalyzer('modBbCodeUsage');
                if ($modBbCodeUsage->tagType)
                {
                    $title = \utf8_ucfirst($modBbCodeUsage->tagType);
                    $tagTitle = $modBbCodeUsage->title;
                    if ($tagTitle)
                    {
                        $title .= ": " . $tagTitle;
                    }

                    $this->addThreadmarkForReply($threadmarkCategoryId, $title);
                }
            }
        }
        finally
        {
            $this->checkForModBbCodeUsage = false;
        }

        return $message;
    }

    protected function canCheckForModBbCodeUsage(int $threadmarkCategoryId): bool
    {
        if (
            !$threadmarkCategoryId
            || !$this->messageEntity->isValidRelation('Threadmark')
        )
        {
            return false;
        }

        /** @var \SV\Threadmarks\Entity\ThreadmarkCategory $threadmarkCategory */
        $threadmarkCategory = $this->em()->find(
            'SV\Threadmarks:ThreadmarkCategory',
            $threadmarkCategoryId
        );

        return ($threadmarkCategory && $threadmarkCategory->canUse());
    }

    protected function addThreadmarkForReply(int $threadmarkCategoryId, string $label)
    {
        \SV\Threadmarks\Globals::$injectThreadmarkIntoReplyCategory = $threadmarkCategoryId;
        \SV\Threadmarks\Globals::$injectThreadmarkIntoReplyLabel = $label;
    }

    /**
     * @noinspection PhpDocMissingThrowsInspection
     * @return \XF\BbCode\Processor
     */
    protected function getBbCodeProcessor()
    {
        $processor = parent::getBbCodeProcessor();

        if ($this->checkForModBbCodeUsage)
        {
            $class = $this->app->extendClass('\SV\AdvancedBbCodesPack\BbCode\ProcessorAction\AnalyzeModBbCodeUsage');
            /** @var \SV\AdvancedBbCodesPack\BbCode\ProcessorAction\AnalyzeModBbCodeUsage $modBbCodeUsage */
            $modBbCodeUsage = new $class();
            $processor->addProcessorAction('modBbCodeUsage', $modBbCodeUsage);
        }

        return $processor;
    }
}